package resmobcity;


import java.util.*;
import java.io.*;

import ufz.aloe.urban.helper.*;
import ufz.aloe.urban.model.*;



/**
 * controls the flow of actions
 * @param parametersRead //are all parameterfiles ok? can the model be run?
 */
public class DemographyTest extends VectorConverter{
	
	private int numberOfTimeSteps = 10;
	private float[] cohortSurvive;
	private float[] cohortZu;
	private float[] cohortWeg;
	private float[] cohortBirth;
	private float[][] cohortToHousehold;
	private float[][] boroughMovement;
	private float[][] householdTypeConstraints;


	public static void main(String[] args) throws IOException {
	
		
	// read parameter files into input vectors
		Vector householdTypeInput=new Vector();
		Vector boroughInput=new Vector();
		Vector boroughMove=new Vector();
		Vector cohort=new Vector();
		Vector initPopulation=new Vector();
		Vector householdTypeConstraints=new Vector();
		
		householdTypeInput = readParameterFiles("householdType.csv");
		boroughInput = readParameterFiles("borough_test.csv");
		boroughMove = readParameterFiles("move_borough_test.csv");
		householdTypeConstraints = readParameterFiles("householdTypeConstraints.csv");
		cohort = readParameterFiles("cohorts.csv");
		initPopulation = readParameterFiles("population.csv");
		
		new DemographyTest(householdTypeInput,
							boroughInput,
							boroughMove,
							cohort,
							householdTypeConstraints,
							initPopulation);
		
	}
	
	public DemographyTest(
				Vector householdTypeInput,
				Vector boroughInput,
				Vector boroughMove,
				Vector cohort,
				Vector householdTypeConst,
				Vector initPopulation) {
		
		
		
		String [] householdTypes; //the household types that are available 
		int [] personsPerHousehold; //the number of persons that belong to a household is a vector of
		int [] maxPersonsPerHousehold;
		int [] minYearMoveArray; 
		
		householdTypes = new String[getStringArray(0,householdTypeInput).length];
		householdTypes = getStringArray(0,householdTypeInput);
		personsPerHousehold = getIntArray(2,householdTypeInput);
		maxPersonsPerHousehold = getIntArray(3,householdTypeInput);
		minYearMoveArray= getIntArray(4,householdTypeInput);
		cohortSurvive = getFloatArray(3,cohort);
		cohortBirth = getFloatArray(4,cohort);
		cohortZu = getFloatArray(5,cohort);
		cohortWeg= getFloatArray(6,cohort);
		cohortToHousehold = getFloatMatrix(7,cohort);
		householdTypeConstraints = getFloatMatrix(1,householdTypeConst);
		
		int [] boroughID;  //list of all boroughs (IDs)
		float []percentVacancy; //  vectors of boroughs
		float [][]percentCohort;  //vector of boroughs, inner vectors of cohort

		boroughID = getIntArray(0,boroughInput);
		boroughMovement = getFloatMatrix(2,boroughMove);
		percentVacancy = getFloatArray(2,boroughInput);
		float[] inhabitantsBorough = getFloatArray(3,boroughInput);
		percentCohort = getFloatMatrix(4,boroughInput);
		float []persYear = getFloatArray(1,initPopulation);
		
		Inhabitant.init(cohortSurvive, cohortBirth, 5);
		Vector[] inhabitants = new Vector[boroughID.length];
		
		
		for (int i=0; i<boroughID.length; i++){
			inhabitants[i] = new Vector();
			float[] cumPercentCohort = new float[percentCohort[i].length];
			/*for(int coh = 0; coh < percentCohort[i].length; coh++) {
				if(coh==0) cumPercentCohort[coh] = percentCohort[i][coh]/100;
				else cumPercentCohort[coh] = cumPercentCohort[coh-1]+percentCohort[i][coh]/100;
				//System.out.println(coh+"    "+cumPercentCohort[coh]);
			}
			for(int inh = 0; inh < inhabitantsBorough[i]; inh++) {
				float ran = (float) Math.random();
				int c=0;
				for(int coh = 0; coh < cumPercentCohort.length; coh++) {
					if(ran <= cumPercentCohort[coh] && ran > ((coh==0) ? 0 : cumPercentCohort[coh-1]) ) {
						c=coh;
						break;
					}
				}
				
				inhabitants[i].addElement(new Inhabitant(c, (int)(Math.random()*5) ) );
			
			}*/
			//for(int inh = 0; inh < inhabitantsBorough[i]; inh++) {
				//System.out.println(inh);
				for (int y=0; y<persYear.length;y++) {
					for (int p=0; p<persYear[y];p++) {
						inhabitants[i].addElement(new Inhabitant( (int) (y/5), y%5 ) );
					}
				}
				
			//}
			
		}
		
		
		
		float[][][] inhabitantsBoroughCohortTimestep 
					= new float[boroughID.length][cohortSurvive.length][numberOfTimeSteps+1];
		
		
		int[][] sum = new int[numberOfTimeSteps+1][cohortSurvive.length+1];
		
		for(int b = 0; b < inhabitants.length; b++) {
			for(int i = 0; i < inhabitants[b].size(); i++) {
				Inhabitant inhab = (Inhabitant) inhabitants[b].elementAt(i);
				inhabitantsBoroughCohortTimestep[b][inhab.getCohort()][0]++;
				sum[0][inhab.getCohort()+1]++;
			}
		}
		
		
		for(int t = 1; t <= numberOfTimeSteps; t++) {
			System.out.println("step "+t);
			sum[t][0]=t;
			Vector[] inhabitantsAdd = new Vector[boroughID.length];
			for(int b = 0; b < inhabitants.length; b++) {
				inhabitantsAdd[b] = new Vector();
				int[] inhabB=new int[cohortSurvive.length];
				float[] inhabZu=new float[cohortSurvive.length];
				float[] inhabWeg=new float[cohortSurvive.length];
				for(int i = 0; i < inhabitants[b].size(); i++) {
					Inhabitant inhab = (Inhabitant) inhabitants[b].elementAt(i);
					inhabB[inhab.getCohort()]++;
					int res = inhab.mature();
					if(res==-1) {
						inhabitants[b].removeElementAt(i);
						i--;
					} 
					else {
						if(res==1) {
							inhabitantsAdd[b].addElement( new Inhabitant( 0, 0 ) );
						}
					}
				}
				for(int i =0; i<inhabZu.length; i++) {
					if(boroughMovement[0].length >= t*2+1 ) {
						inhabZu[i]=(cohortZu[i]*boroughMovement[b][t*2])/100;
						inhabWeg[i]=((cohortWeg[i]*boroughMovement[b][t*2+1])/100)/inhabB[i];		
					} 
					else {
						inhabZu[i]=(cohortZu[i]*boroughMovement[b][boroughMovement[0].length-2])/100;
						inhabWeg[i]=((cohortWeg[i]*boroughMovement[b][boroughMovement[0].length-2])/100)/inhabB[i];
					}
					
					//System.out.println(inhabZu[i]+"   "+inhabWeg[i]);
				}
			//	System.out.println(boroughMovement[b][t*2+1]);
			//	System.out.print(inhabitants[b].size()+"   ");
				for(int rem =0; rem<inhabitants[b].size(); rem++) {
					Inhabitant inhab = (Inhabitant) inhabitants[b].elementAt(rem);
					
					if( Math.random() <= inhabWeg[inhab.getCohort()] ) {
						inhabitants[b].removeElementAt(rem);
						rem--;
					}
				}
			//	System.out.println(inhabitants[b].size());
				//System.out.print(inhabitantsAdd[b].size()+"   ");
				for(int i =0; i<inhabZu.length; i++) {
					for(int add =0; add<inhabZu[i]; add++) 
							inhabitantsAdd[b].addElement(new Inhabitant(i, (int)(Math.random()*5) ) );
					
				}
				//System.out.println(inhabitantsAdd[b].size());
				for(int a = 0; a < inhabitantsAdd[b].size(); a++) {
					inhabitants[b].addElement(inhabitantsAdd[b].elementAt(a));
				}
			}
			for(int b = 0; b < inhabitants.length; b++) {
				for(int i = 0; i < inhabitants[b].size(); i++) {
					Inhabitant inhab = (Inhabitant) inhabitants[b].elementAt(i);
					inhabitantsBoroughCohortTimestep[b][inhab.getCohort()][t]++;
					sum[t][inhab.getCohort()+1]++;
				}
			}
			int[][] out = new int[cohortSurvive.length+1][inhabitants.length];
			
			for(int b =0; b<out[0].length; b++) {
				for(int c = 0; c<out.length; c++) {
					if(c==0) {
						out[c][b] = boroughID[b];
					}
					else {
						out[c][b] = (int) inhabitantsBoroughCohortTimestep[b][c-1][t];
					}
				}
			}
			writeTable(out, new File("demography_out_"+t+".csv") );
		}
		writeTable(sum, new File("demography_out_city.csv") );
		
		
	}

	public static Vector readParameterFiles(String inputFile){
		Vector inputVec = new Vector();
		try {
			/**
			 * Reads the parameter files in the input folder
			 *  @param inputFile String that conatins the path to the parameter file
			 */
			
			FileReader in = new FileReader(inputFile);
			CsvReader csv = new CsvReader(in);
			inputVec = csv.readCsvFile();

		//	System.out.println(inputVec.size());
		//	System.out.println(inputVec);
		
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return inputVec;
	}
	public static void printMatrix(int[][] m) {
		for(int i = 0; i < m.length; i++) {
			for(int j = 0; j < m[0].length; j++) {
				System.out.print(m[i][j]+"  ");
			}
			System.out.println("");
		}
	}
	public static void printArray(int[] m) {
		for(int i = 0; i < m.length; i++) {
			System.out.print(m[i]+"  ");
		}	
		System.out.println("");
	}
	public static void writeTable(int[][] d, File f) {
			StringWriter buffer = new StringWriter();
			PrintWriter out = new PrintWriter( buffer );
			for(int i = 0; i < d.length; i++) {	
				for(int j = 0; j < d[0].length; j++) {	
					out.print(d[i][j]+";");
				}
				out.println("");
			}
			
	    	String toWrite = buffer.toString();
	
	    	
	    	try {
	    		FileWriter fw = new FileWriter( f );
	    		fw.write( toWrite );
	    		fw.close();
	    	} catch ( IOException e ) {
	      		System.out.println( "Konnte Datei nicht erstellen" );
	    	}
	  
	}

}
