package ufz.aloe.urban.controler;


import java.util.*;
import java.io.*;

import ufz.aloe.urban.helper.*;
import ufz.aloe.urban.model.*;

import sa.data.*;
import sa.io.AsciiFileReader;


/**
 * controls the flow of actions
 * @param parametersRead //are all parameterfiles ok? can the model be run?
 */
public class UrbanModelControler {

	public static void main(String[] args) throws IOException {
	
		
		
	// read parameter files into input vectors
		Vector structureTypeInput=new Vector();
		Vector householdTypeInput=new Vector();
		//Vector minYearMove = new Vector();
		Vector boroughInput=new Vector();
		Vector attractivenessInput=new Vector();
		Vector attractivenessStructureType=new Vector();
		Vector attractivenessInfra=new Vector();
		
		Vector landParcelInput=new Vector();
		Vector maximumPropertyDistances=new Vector();
		Vector weights=new Vector();
		Vector cohort=new Vector();
		Vector householdTypeConstraints=new Vector();
		Vector percentCohortsMoveIn=new Vector();
		Vector buildings=new Vector();
		Vector schools=new Vector();
		Vector shopping=new Vector();
		Vector budget=new Vector();
		Vector attractivenessGreenery=new Vector();
		
		
		Raster landParcelIDRaster = new AsciiFileReader().readAsciiFile(Properties.FILE_LANDPARCELID, true);
		Raster landParcelDist = new AsciiFileReader().readAsciiFile(Properties.FILE_LANDPARCELDIST, true);
		Raster greenery = new AsciiFileReader().readAsciiFile(Properties.FILE_GREENERY, true);
		
		
		

		
		// defines use type names, ids and flats per building
		structureTypeInput = readParameterFiles(Properties.FILE_STRUCTURETYPE);
		// defines household type names, ids and person per household
		householdTypeInput = readParameterFiles(Properties.FILE_HOUSEHOLDTYPE);
		// defines borough id, name and household composition
		boroughInput = readParameterFiles(Properties.FILE_BOROUGH);
		// ???
		attractivenessInput = readParameterFiles(Properties.FILE_ATTRACT_POPDENS);
		attractivenessStructureType = readParameterFiles(Properties.FILE_ATTRACT_STRUCTTYPE);
		attractivenessInfra = readParameterFiles(Properties.FILE_ATTRACT_INFRASTRUCTURE);
		attractivenessGreenery = readParameterFiles(Properties.FILE_ATTRACT_GREENERY);
		
		
		// defines landparcels with id, area, numberOfBuildings, initialStructureType, borough
		landParcelInput = readParameterFiles(Properties.FILE_LANDPARCEL_INPUT);
		maximumPropertyDistances = readParameterFiles(Properties.FILE_MAX_PROPERTYDISTANCE);
		householdTypeConstraints = readParameterFiles(Properties.FILE_HOUSEHOLDTYPE_CONSTRAINTS);
		weights = readParameterFiles(Properties.FILE_WEIGHTS_PARAMETERS);
		cohort = readParameterFiles(Properties.FILE_COHORTS);
		percentCohortsMoveIn = readParameterFiles(Properties.FILE_MOVEIN);
		buildings = readParameterFiles(Properties.FILE_HOUSES_OUT);
		schools = readParameterFiles(Properties.FILE_SCHOOLS);
		shopping = readParameterFiles(Properties.FILE_SHOPPING);
		budget = readParameterFiles(Properties.FILE_BUDGET);
		
		
		int nrST=((Vector)structureTypeInput.elementAt(0)).size()-1;
		System.out.println(nrST);
		Raster[] costs= new Raster[nrST];
		
		for ( int i = 0; i < nrST; i++ ) {
			costs[i]=new AsciiFileReader().readAsciiFile(Properties.PATH_STRUCTTYPES +(i+1)+".asc", true);
		}
		
		
		
		// create city object
		City leipzig = new City();
		
	
		//pass input vector to the city object, which creates the landparcels, householdTypes, households
		//structure types, building units for leipzig
		leipzig.instantiateModel(
							landParcelIDRaster,
							costs,
							structureTypeInput,
							householdTypeInput,
							//minYearMove,
							boroughInput,
							attractivenessInput,
							attractivenessStructureType,
							attractivenessInfra,
							landParcelInput, 
							landParcelDist.getFloatMatrix(), 
							maximumPropertyDistances,
							weights,
							cohort,
							householdTypeConstraints,
							percentCohortsMoveIn,
							buildings,
							schools,
							shopping,
							budget,
							greenery,
							attractivenessGreenery);
		leipzig.inputOutput(); // in order to check the code
		runModel(leipzig);
	
	}
		
	public static void runModel(City city) {
		int[][] dem = new int[Properties.NUMBER_OF_TIMESTEPS][0];
		int[][] hht = new int[Properties.NUMBER_OF_TIMESTEPS][0];
		city.calculateAttractiveness();
		for (int i=0; i<Properties.NUMBER_OF_TIMESTEPS; i++){
			System.out.println("run: "+i);
			
			city.outputPerTimeStep(i);
			
			city.resetMigration();
			
			city.findBuildingsToRemove();
		
			city.calculateAttractiveness();
			//city.sortLandParcelsByAttractiveness();
			city.extractVacantFlats(true);
			city.matureHouseholds();
			
			city.externalMoveHouseholds();
			
			city.moveHouseholds();
			
			city.removeBuildings();
			
			dem[i]=city.getCohorts();
			hht[i]=city.countHouseholds();
			
			
			//System.out.println()
			//printMatrix( city.getLandParcelArray()[0].getHouseholdTypesInStructureTypes() );
		}
		city.outputPerTimeStep(Properties.NUMBER_OF_TIMESTEPS);
		
		writeTable(dem,new File("results/cohorts_out.csv"));
		writeTable(hht,new File("results/households_out.csv"));
	}

	public static Vector readParameterFiles(String inputFile){
		Vector inputVec = new Vector();
		try {
			/**
			 * Reads the parameter files in the input folder
			 *  @param inputFile String that conatins the path to the parameter file
			 */
			
			FileReader in = new FileReader(inputFile);
			CsvReader csv = new CsvReader(in);
			inputVec = csv.readCsvFile();

		//	System.out.println(inputVec.size());
		//	System.out.println(inputVec);
		
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return inputVec;
	}
	public static void printMatrix(int[][] m) {
		for(int i = 0; i < m.length; i++) {
			for(int j = 0; j < m[0].length; j++) {
				System.out.print(m[i][j]+"  ");
			}
			System.out.println("");
		}
	}
	public static void printArray(int[] m) {
		for(int i = 0; i < m.length; i++) {
			System.out.print(m[i]+"  ");
		}	
		System.out.println("");
	}
	public static void printArray(float[] m) {
		for(int i = 0; i < m.length; i++) {
			System.out.print(m[i]+"  ");
		}	
		System.out.println("");
	}
	public static void writeTable(int[][] d, File f) {
			StringWriter buffer = new StringWriter();
			PrintWriter out = new PrintWriter( buffer );
			for(int i = 0; i < d.length; i++) {	
				for(int j = 0; j < d[0].length; j++) {	
					out.print(d[i][j]+";");
				}
				out.println("");
			}
			
	    	String toWrite = buffer.toString();
	
	    	
	    	try {
	    		FileWriter fw = new FileWriter( f );
	    		fw.write( toWrite );
	    		fw.close();
	    	} catch ( IOException e ) {
	      		System.out.println( "Konnte Datei nicht erstellen" );
	    	}
	  
	}
	public static void writeTable(float[][] d, File f) {
			StringWriter buffer = new StringWriter();
			PrintWriter out = new PrintWriter( buffer );
			for(int i = 0; i < d.length; i++) {	
				for(int j = 0; j < d[0].length; j++) {	
					out.print(d[i][j]+";");
				}
				out.println("");
			}
			
	    	String toWrite = buffer.toString();
	
	    	
	    	try {
	    		FileWriter fw = new FileWriter( f );
	    		fw.write( toWrite );
	    		fw.close();
	    	} catch ( IOException e ) {
	      		System.out.println( "Konnte Datei nicht erstellen" );
	    	}
	  
	}

}
