//see https://eclipse-tutorial.dev.java.net/eclipse-tutorial/german/part1.html how to put external jar files to the classpath

package ufz.aloe.urban.helper; 
import java.io.*;
import java.util.Vector;

import org.apache.regexp.*; //from http://jakarta.apache.org/regexp/

public class CsvReader {
	/*
	  *  first pattern matches quoted fields, second unqouted, the third null fields
	  */
		private static final String CSV_PATTERN =
			"\"([^\"\\\\]*(\\\\.[^\"\\\\]*)*)\";?|([^;]+);?|;";
			// there are three parts of the expression, separetd by | wich stands for a logical OR
			//the last one simply matches a , which is the field separator of csv files
			// the second one matches every thing that is not a comma, i.e. it matches every thing between two commas
			// the first one matches everything that:
			// a) starts with an " and ends with an " - the backslash is needed to mask the quotes
			// b) ...
		private BufferedReader is;
		private Vector fields = new Vector();
		private RE csv;
		
		public CsvReader(FileReader fileReader){
			is = new BufferedReader (fileReader);
			csv = new RE(CSV_PATTERN);
		}
		public Vector readCsvFile() throws IOException {
			String line;
			Vector rowVector ;
			int numberFields = 0;
			int count = 0;
			boolean firstLine = true;
			long lineNumber = 0;
			// for each line
			while ( (line = is.readLine() ) != null) {
				//System.out.println("line = '"+ line + "'");
				count = 0;
				lineNumber++;
				rowVector =  new Vector();
				
				// for each field
				for (int fieldNum = 0, offset = 0; csv.match(line, offset); fieldNum++){
					//print the field (0=null, 1 = quoted, 3=unquoted)
					int n = csv.getParenCount()-1; //Returns the number of parenthesized subexpressions available after a successful match.
					if (n==0) { //null field
						// System.out.println("field[" + fieldNum + "] = `'");
						rowVector.add("");
					}
					else {
						// System.out.println("field[" + fieldNum + "] = '" + csv.getParen(n) + "'");
					rowVector.add(csv.getParen(n));
					
					//skip what has already matched
					offset +=csv.getParen(0).length();
					count ++;
					
					}
				}
				if (firstLine){//firste line is read
					numberFields = count;
					firstLine = false;
				}
				else {
					if (count != numberFields) {
						throw new IllegalArgumentException(
		            "readCsvFile: number of fields does not match! Line Number: " + lineNumber + 
		             " file: " + is.toString());
						//is.close();
					}
				}
				
			fields.add(rowVector);
			
			}
			return fields;
		}
}
