package ufz.aloe.urban.helper;

import java.io.File;
import java.io.IOException;
import java.util.Date;

/**
 * Static helper class for file operations
 * 
 */
public class FileHelper {

	public static String newline = System.getProperty("line.separator");
	public static String seperator = System.getProperty("file.separator");
	
	
	/**
	 * @param aFilename the name of a file (fullname including the path)
	 * @return true if aFilename is a file
	 * @throws IOException
	 */
	public static boolean testFile(String aFilename)throws IOException{
		File f;
		if (aFilename == "")
     		return false;
     	else {
	     	
	     		f = new File(aFilename);
	     		if (!f.exists() && !f.isFile())
	     			return false;
	     		else 
	         		return true;
	     	}
		
     	
     }
	/** 
	 * @param aFoldername the name of a folder (fullname including the path)
	 * @return true if aFoldername is a folder
	 * @throws IOException
	 */
	public static boolean testFolder(String aFoldername)throws IOException{
		File f;
		if (aFoldername == "")
     		return false;
     	else {
	     	
     		f = new File(aFoldername);
     		if (!f.exists() && !f.isDirectory())
     			return false;
     		else 
         		return true;
     	}
	}
	
	/**
	 * @param aFilename the Name of a file(fullname including the path)
	 * @throws IOException
	 */
	public static String status(String aFilename) throws IOException{
		File f;
		String fileStatus ="";
		Date d = new Date();
		
		f = new File(aFilename);
		
		if (!f.exists()) {
			return "File: " + aFilename + " not found!";
		}
		
		fileStatus += newline + "Connonical name " + f.getCanonicalPath();
		
		fileStatus += newline + "Parent directory: " + f.getParent();
		
		if (f.canRead()){
			fileStatus += newline + "File is readable";
		}
		
		if (f.canWrite()){
			fileStatus += newline + "File is writable";
		}
		
		d.setTime(f.lastModified());
		
		fileStatus += newline + "Last modified: " + d;
		
		if (f.isFile()){
			fileStatus += newline + "File size is " + f.length() + " bytes";
		}
		else if (f.isDirectory()){
			fileStatus += newline + "It is a directory.";
		}
		else {
			fileStatus += newline + "I dunno! Neither a file nor a directory!";
		}
			
		
		return fileStatus;
		
	}
	
	public boolean createNewFile(String aFilename, boolean overwrite) throws IOException{
		File f;
		if (!testFile(aFilename) || overwrite){
			//TODO ensure parent folder is valid
			f = new File(aFilename);
			return f.createNewFile();
		}
		else
			return false;
	}
	/** Returns a string array of all filenames of parameter files in the folder
	 * @param aFolder the folder to search in
	 * @return A string array with the file names of the parameter files in the folder
	 * @throws IOException
	 */
	public String [] listParameterFiles(String aFolder) throws IOException{
		File f;
		ParameterFileFilter pff = new ParameterFileFilter();
		String [] dir = null;
		
		if (testFolder(aFolder)) {
			f = new File(aFolder);
			dir = f.list(pff);
		}
		return dir;
		 
		
	}
}
