package ufz.aloe.urban.model;

import java.util.Vector;
import java.awt.Point;
import sa.data.Raster;

public class Building extends Point {
	
	private float ID;
	private LandParcel parcel;
	private StructureType structureType;
	private Flat flat;
	private int numberOfFlats;
	private Vector<Household> householdVector;
	private int borough;
	private float[] attractiveness;
	private int cellX=0;
	private int cellY=0;
	private boolean toBuy=false;
	private int cost;
	private float[] greeneryProportion;
	private int yearsOfVacancy=0;
	
	public Building(int x, int y, 
					Raster cost,
					LandParcel parcel, 
					StructureType structureType, 
					float id, 
					Flat flat, 
					int numFlats,
					boolean toBuy,
					Raster greenery,
					int maxRangeGreenery,
					int greeneryClasses) {
		this.parcel=parcel;
		this.structureType=structureType;
		ID=id;
		this.flat=flat;
		flat.setBuilding(this);
		numberOfFlats=numFlats;
		householdVector=new Vector<Household>();
		this.x=x;
		this.y=y;
		int[] rel=cost.getCoordsRelative(x,y);
		this.toBuy=toBuy;
		cellX=rel[0];
		cellY=rel[1];
		calcGreeneryProportions(greenery, cellX, cellY, maxRangeGreenery, greeneryClasses);
		int c=(int) cost.valueAt(cellX, cellY);
		if(toBuy) {
			this.cost = c%10;
		}
		else {
			this.cost = c/10;
		}
	}	
	public int getNumberOfFlats() {
		return numberOfFlats;
	}
	public int getVacancy() {
		return numberOfFlats-householdVector.size();
	}
	public float getPercentVacancy() {
		return (float)getVacancy()/(float)getNumberOfFlats();
	}
	public int getCost() {
		return cost;
	}
	public Vector<Household> getHouseholds() {
		return householdVector;
	}
	/*public void calculateAttractiveness(
								HouseholdType[] householdTypeArray, 
								float[] weightsOfParameters){
		flat.calculateAttractiveness(householdTypeArray, weightsOfParameters);
	}*/
	public StructureType getStructureType() {
		return structureType;
	}
	public int getStructureTypeID() {
		return structureType.getID();
	}
	public int getLandParcelID() {
		return parcel.getID();
	}
	public boolean remove(int minYearsVacant) {
		return (yearsOfVacancy>=minYearsVacant);
	}
	public void updateNextStep(float percentVacancy) {
		if(getPercentVacancy()>=percentVacancy) yearsOfVacancy++;
		else yearsOfVacancy=0;
	}
	public int getBoroughID() {
		return parcel.getBorough();
	}
	public Flat getFlat() {
		return flat;
	}
	public float getID() {
		return ID;
	}
	public boolean addHousehold(Household household) {
		if(householdVector.size()<numberOfFlats) {
			householdVector.add(household);
			parcel.migrateIn(household);
			return true;
		}
		return false;
	}
	public void removeHousehold(Household household) {
		householdVector.remove(household);
		parcel.migrateOut(household);
	}
	public float[] getAttractiveness() {
		return flat.getAttractiveness();
	}
	
	public void calcGreeneryProportions(Raster green, int x, int y, int maxRangeGreenery, int greeneryClasses) {
		greeneryProportion=new float[greeneryClasses];
		int range=(int) (maxRangeGreenery/green.cellsize());
		float count=0;
		for(int i=x-range;i<=x+range;i++) {
			for(int j=y-range;j<=y+range;j++) {
				if( Math.hypot(x-i,y-j) <= range && green.inExtend(i,j) ) {
					int c=(int) green.valueAt(i,j);
					if(c>0) greeneryProportion[c-1]++;
					count++;
				}
			}	
		}
		for(int i=0;i<greeneryProportion.length;i++) {
			greeneryProportion[i] /= count;
			//System.out.print(greeneryProportion[i]+"  ");
		}
		//System.out.println();
	}
	
	
	public void calculateAttractiveness(
							HouseholdType[]  householdTypeArray, 
							LandParcel[] landParcelArray, 
							float[][] landParcelDistances,  
							Vector<Infrastructure> infrastructure,
							float[] maximumPropertyDistances,
							float[][] weightsOfParameters){
		attractiveness = new float[householdTypeArray.length];
		float inhabitantsPerAreaSum = 0;
		float a = 0;
		
		
		Vector<Building> buildings=new Vector<Building>();
		
		
		for (int j =0; j < landParcelArray.length; j++){
			if(landParcelDistances[parcel.getID()][landParcelArray[j].getID()] 
					<= maximumPropertyDistances[City.SOCIAL_MILIEU]) {
				{
					Vector<Building> bb=landParcelArray[j].getBuildings();
					for (Building b : bb){
						if(b.distance(this)<=maximumPropertyDistances[City.SOCIAL_MILIEU]) {
							buildings.add(b);
						}
					}
					
				}
			}
		}
					
		double area=Math.pow(maximumPropertyDistances[City.SOCIAL_MILIEU], 2)*Math.PI;
		
		for (int j =0; j < householdTypeArray.length; j++){
			int nrhh=0;
			for (Building b : buildings){
				Vector<Household> hh=b.getHouseholds();
				Vector<Household> rem=new Vector<Household>();
				for (Household h : hh){
					if(h.getHouseholdType()==null) rem.add(h);
					else {
						nrhh++;
						int hhID=h.getHouseholdTypeID()-1;
						attractiveness[j] += 
							 householdTypeArray[j].getAttractivenessWeightsPopDens ()[hhID];
					
					}
				}
				hh.removeAll(rem);
			}
			float nearestSchool=maximumPropertyDistances[City.TYPE_SCHOOL];
			float nearestShopping=maximumPropertyDistances[City.TYPE_SHOPPING];
			
			//if(weightsOfParameters[City.TYPE_SCHOOL][j]>0) System.out.println("_ Infrastructure: "+infrastructure.size());
			
			
			for (Infrastructure inf : infrastructure){
				
				if(inf.getType()==City.TYPE_SCHOOL) {
					//if(weightsOfParameters[City.TYPE_SCHOOL][j]>0) System.out.println("school;   x: "+inf.getX()+",  y: "+inf.getY());
			
					if(inf.distance(this)<nearestSchool) {
						nearestSchool=(float)inf.distance(this);
					}
				}
				if(inf.getType()==City.TYPE_SHOPPING) {
					//if(weightsOfParameters[City.TYPE_SCHOOL][j]>0) System.out.println("shopping;   x: "+inf.getX()+",  y: "+inf.getY());
					
					if(inf.distance(this)<nearestShopping) {
						nearestShopping=(float)inf.distance(this);
					}
				}
			}
			
			//if(weightsOfParameters[City.TYPE_SCHOOL][j]>0) System.out.println("_ nearestSchool: "+nearestSchool+";   nearestShopping: "+nearestShopping);
			
			nearestSchool = (maximumPropertyDistances[City.TYPE_SCHOOL]-nearestSchool) / maximumPropertyDistances[City.TYPE_SCHOOL];
			nearestShopping = (maximumPropertyDistances[City.TYPE_SHOPPING]-nearestShopping) / maximumPropertyDistances[City.TYPE_SHOPPING];
			
			
			
			//if(weightsOfParameters[City.TYPE_SCHOOL][j]>0) System.out.println("  nearestSchool: "+nearestSchool+";   nearestShopping: "+nearestShopping);
			
			if(nrhh>0) attractiveness[j] = weightsOfParameters[City.SOCIAL_MILIEU][j]*(attractiveness[j]/nrhh);
			attractiveness[j] += weightsOfParameters[City.STRUCTURETYPE][j] * householdTypeArray[j].getAttractivenessStructureType()[structureType.getID()-1];
			attractiveness[j] += weightsOfParameters[City.TYPE_SCHOOL][j] * nearestSchool * householdTypeArray[j].getAttractivenessInfrastructure()[City.TYPE_SCHOOL];
			attractiveness[j] += weightsOfParameters[City.TYPE_SHOPPING][j] * nearestShopping * householdTypeArray[j].getAttractivenessInfrastructure()[City.TYPE_SHOPPING];
			for(int i=0;i<greeneryProportion.length;i++) {
				attractiveness[j] += weightsOfParameters[City.TYPE_GREENERY][j] * greeneryProportion[i] * householdTypeArray[j].getAttractivenessGreenery()[i];
			}	
			//inhabitantsPerAreaSum +=  inhabitantsPerArea[j];
			//a +=attractiveness[j] * inhabitantsPerArea[j];
		}
		
		
		
		flat.setAttractiveness(attractiveness);
	}
	public int[] getHouseholdTypes(HouseholdType[] householdTypeArray) {
		Vector<Household> hh=getHouseholds();
		int[] ret=new int[householdTypeArray.length];
		Vector<Household> rem=new Vector<Household>();
		for (Household h : hh){
			if(h.getHouseholdType()==null) rem.add(h);
			else {
				ret[h.getHouseholdTypeID()-1]++;
			}
		}
		hh.removeAll(rem);
		return ret;
	}
	
	
	public int[] getCohorts(int[] cohortArray) {
		Vector<Household> hh=getHouseholds();
		int[] ret=new int[cohortArray.length];
		Vector<Household> rem=new Vector<Household>();
		for (Household h : hh){
			if(h.getHouseholdType()==null) rem.add(h);
			else {
				Vector<Inhabitant> inh=h.getInhabitants();
				for(Inhabitant in : inh) {
					ret[in.getCohort()]++;
				}
				
			}
		}
		hh.removeAll(rem);
		return ret;
	}
	public boolean toBuy(){
		return toBuy;
	}
	public int cost(){
		return cost;
	}
}
