package ufz.aloe.urban.model;

import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Observable;
import java.util.Vector;
import java.util.Random;
import sa.data.Raster;
import java.awt.Point;

import com.neilmoomey.util.*;

/**
 * @author duethman
 *
 */
public class City  extends Observable  {
	
	public final static int SOCIAL_MILIEU = 0;
	//public final static int SUM_LANDPARCEL_IN_FLAT = 1;
	public final static int STRUCTURETYPE = 1;
	public final static int TYPE_SCHOOL = 2;
	public final static int TYPE_SHOPPING = 3;
	public final static int TYPE_GREENERY = 4;
	
	public final static int GRUNDSCHULE = 1;
	public final static int MITTELSCHULE = 2;
	public final static int GYMNASIUM = 3;
	
	private StructureType[]  structureTypeArray;	
	private HouseholdType[]  householdTypeArray;
	private Borough[]  boroughArray;	
	private LandParcel[]  landParcelArray;	
	//private Household [] householdArray;
	private Vector<Household> allHouseholdsOfTheCity;
	private Vector<Household> newHouseholdsOfTheCity;
	
	private Vector<Household> householdsMoveIn;
	
	private Vector<Building> buildingsToRemove;
	
	private LandParcel[][] landParcelArraySort;
	private float[][] landParcelDistances;
	private float[] cohortSurvive;
	private float[] cohortBirth;
	private float[][] cohortToHousehold;
	private float[][] householdTypeConstraints;
	private float[] maximumPropertyDistances;
	private float[][] weightsOfParameters;
	private float[] thresholdMoving;  // Array of HouseholdType
	
	private float[] percentCohortMoveIn;
	private int numMovingIn;
	private float[] percentCohortMoveOut;
	private int numMovingOut;
	private Vector<Move> movingLastStep;
	private int[][] householdTypeChanges;
	private Raster landParcelId;
	
	/**
	 * Array of budget distribution, first index budget class, secound index household type
	 */
	private float[][] budget;     
	
	
	private Vector<Flat> vacantFlats;
	private Vector<Integer> numOfVacantFlats;
	//private int[] cohorts;
	//private float[] expectedHouseholds;
	//private int[] countedHouseholds;
	private float[] meanHouseholdSize;
	private Random random=new Random();
	private Vector<Infrastructure> infrastructure;
	//private int[][] demography;
	
	
	private int minYearsVacant=5;
	
	
	public void instantiateModel(
				Raster landParcelId,
				Raster[] costs,
				Vector structureTypeInput,
				Vector householdTypeInput,
				//Vector minYearMove,
				Vector boroughInput,
				Vector attractivenessInput,
				Vector attractivenessStructureType,
				Vector attractivenessInfra,
				Vector landParcelInput,
				float[][] landParcelDistances, 
				Vector maximumPropertyDistances,
				Vector weights,
				Vector cohort,
				Vector householdTypeConst,
				Vector percentCohortMoveIn,
				Vector buildingPos,
				Vector schoolPos,
				Vector shoppingPos,
				Vector budgetInput,
				Raster greenery,
				Vector attGreenery)
			throws UnequalInputArrayLengthException{
		
	//UseTypes
		this.numMovingIn=numMovingIn;
		this.landParcelId=landParcelId;
		String [] structureTypes; //the use types that are available 
		int [] flatsPerBuilding; //the number of flats that a building has is a vector of structure type
		float[] proportionToBuy;
		float[] meanRadius;
		structureTypes = getStringArray(0,structureTypeInput);
		flatsPerBuilding = getIntArray(2,structureTypeInput);
		proportionToBuy = getFloatArray(5,structureTypeInput);
		meanRadius = getFloatArray(6,structureTypeInput);
		
		if (structureTypes.length == flatsPerBuilding.length){
			structureTypeArray = new StructureType[structureTypes.length];
			for (int i=0; i<structureTypes.length; i++) {
				StructureType b = new StructureType(structureTypes[i], flatsPerBuilding [i], i+1, meanRadius[i]);
				structureTypeArray[i] = b;
			}
		}
		else 
			throw new UnequalInputArrayLengthException("Problems with input arrays for use types");
	
		
	//HouseholdTypes	
		String [] householdTypes; //the household types that are available 
		int [] personsPerHousehold; //the number of persons that belong to a household is a vector of
		int [] maxPersonsPerHousehold;
		int [] minYearMoveArray; 
		float [][]attractivenessWeightsPopDens; /* vector of householdTypes, inner vectors contain
		the weights the household type gives to population density of the different household types  */
		float [][]attractivenessStructureTypeArray; /* vector of householdTypes, inner vectors contain
		the weights the household type gives to structure types*/
		float [][]attractivenessInfrastructure;
		float [][]attractivenessGreenery;
		
		householdTypes = new String[getStringArray(0,householdTypeInput).length];
		householdTypes = getStringArray(0,householdTypeInput);
		personsPerHousehold = getIntArray(2,householdTypeInput);
		maxPersonsPerHousehold = getIntArray(3,householdTypeInput);
		minYearMoveArray= getIntArray(4,householdTypeInput);
		thresholdMoving = getFloatArray(5,householdTypeInput);
		attractivenessWeightsPopDens = getFloatMatrix(1,attractivenessInput);	
		attractivenessStructureTypeArray = getFloatMatrix(1,attractivenessStructureType);	
		attractivenessInfrastructure = getFloatMatrix(1,attractivenessInfra);	
		attractivenessGreenery = getFloatMatrix(1,attGreenery);	
		
		
		budget = getFloatMatrix(1,budgetInput);	
		
		
		weightsOfParameters= getFloatMatrix(1,weights);
		
		cohortSurvive = getFloatArray(3,cohort);
		cohortBirth = getFloatArray(4,cohort);
		cohortToHousehold = getFloatMatrix(7,cohort);
		householdTypeConstraints = getFloatMatrix(1,householdTypeConst);
		
		
		this.maximumPropertyDistances = getFloatArray(1,maximumPropertyDistances);
		
		//System.out.println("d-max: "+this.maximumPropertyDistances[TYPE_SCHOOL]);
		
		
		float[][] move = getFloatMatrix(1,percentCohortMoveIn);
		this.percentCohortMoveIn = new float[move.length-1];
		this.percentCohortMoveOut = new float[move.length-1];
		
		for(int i=0;i<this.percentCohortMoveIn.length;i++) {
			this.percentCohortMoveIn[i]=move[i][0];
			this.percentCohortMoveOut[i]=move[i][1];
		}
		this.numMovingIn=(int)move[move.length-1][0];
		this.numMovingOut=(int)move[move.length-1][1];
		
		int [] landParcelID; //list of all land parcels in the model (IDs)
		float []area;
		int []numberOfBuildings;	
		int []initialStructureType;
		int []borough;
		float[][] initialHouseholdTypeProportions;
		int[][] buildingPositions;
		
		infrastructure=new Vector<Infrastructure>();
		int[][] schoolPositions;
		int[][] shoppingPositions;
		
		landParcelID = getIntArray(0,landParcelInput);
		area = getFloatArray(1,landParcelInput);
		numberOfBuildings = getIntArray(2,landParcelInput);	
		initialStructureType = getIntArray(3,landParcelInput);	
		borough = getIntArray(4,landParcelInput);
		buildingPositions=getIntMatrix(1,buildingPos);
		schoolPositions=getIntMatrix(1,schoolPos);
		shoppingPositions=getIntMatrix(1,shoppingPos);
		
		
		for(int i = 0; i < schoolPositions[0].length; i++) {
			infrastructure.add(new Infrastructure(
									schoolPositions[1][i],
									schoolPositions[2][i],
									TYPE_SCHOOL,
									schoolPositions[0][i]
								));
		}
		
		for(int i = 0; i < shoppingPositions[0].length; i++) {
			infrastructure.add(new Infrastructure(
									shoppingPositions[0][i],
									shoppingPositions[1][i],
									TYPE_SHOPPING,
									0
								));
		}
		
		//initialHouseholdTypeProportions = getFloatMatrix(5,landParcelInput);
		
		
		
		Inhabitant.init(cohortSurvive, cohortBirth, 5);
		HouseholdType.init(householdTypeConstraints, personsPerHousehold, maxPersonsPerHousehold);
		
		
		if (householdTypes.length == personsPerHousehold.length && 
				householdTypes.length == maxPersonsPerHousehold.length && 
				householdTypes.length == attractivenessWeightsPopDens.length && 
				householdTypes.length == attractivenessWeightsPopDens[0].length  &&
				householdTypes.length == attractivenessStructureTypeArray.length  &&
				householdTypes.length == attractivenessInfrastructure.length  &&
				householdTypes.length == attractivenessGreenery.length  &&
				structureTypeArray.length == attractivenessStructureTypeArray[0].length &&
				householdTypes.length == minYearMoveArray.length &&
				householdTypes.length == cohortToHousehold[0].length ) 
		{
			householdTypeArray = new HouseholdType[householdTypes.length];
			for (int i=0; i<householdTypes.length; i++){
				HouseholdType b = new HouseholdType(
											householdTypes[i], 
											i+1, 
											personsPerHousehold[i],
											attractivenessWeightsPopDens[i], 
											attractivenessStructureTypeArray[i],
											attractivenessInfrastructure[i],
											attractivenessGreenery[i],
											minYearMoveArray[i]);
				householdTypeArray[i] = b;
			}
		}
		else 	
			throw new UnequalInputArrayLengthException("Problems with input arrays for household types");
		
		Household.init(householdTypeArray, budget);

	//borough
		int [] boroughID;  //list of all boroughs (IDs)
		float []percentVacancy; //  vectors of boroughs
		float [][]percentCohort;  //vector of boroughs, inner vectors of householdType

		boroughID = getIntArray(0,boroughInput);
		percentVacancy = getFloatArray(2,boroughInput);
		float[] inhabitantsBorough = getFloatArray(3,boroughInput);
		percentCohort = getFloatMatrix(4,boroughInput);
		
		Vector[] inhabitants = new Vector[boroughID.length];
		Vector[] allHouseholdsOfBorough = new Vector[boroughID.length];
		allHouseholdsOfTheCity= new Vector<Household>();
		
		if (boroughID.length == percentVacancy.length 
				&& boroughID.length == percentCohort.length){ 
			boroughArray = new Borough[boroughID.length];
			for (int i=0; i<boroughID.length; i++){
				inhabitants[i]=createInhabitants( percentCohort[i], (int)inhabitantsBorough[i] );
				
				Borough b = new Borough(boroughID[i], percentVacancy [i],
						percentCohort[i], cohortToHousehold, householdTypeArray);
				boroughArray[i] = b;
				System.out.println(i+"      "+inhabitants[i].size());
			}
			
			
			for (int i=0; i<inhabitants.length; i++) { 
				allHouseholdsOfBorough[i] = sortToHouseholds(inhabitants[i], cohortToHousehold);
				System.out.println(i);
			}
		}
		else 
			throw new UnequalInputArrayLengthException("Problems with input arrays for boroughs");
		
		// rename incomplete households
		
		
		for (int i=0; i<allHouseholdsOfBorough.length; i++) {
			for(int hhb=0; hhb<allHouseholdsOfBorough[i].size(); hhb++) {
				Household hhcurr=((Household) allHouseholdsOfBorough[i].elementAt(hhb));
				if( !hhcurr.isComplete() ) {
					hhcurr.deriveHouseholdType();
				}
				
			}
		}
		
		
		
		
		this.landParcelDistances=landParcelDistances;
		
		
		
		if (landParcelID.length == area.length && 
			landParcelID.length == numberOfBuildings.length && 
			landParcelID.length == initialStructureType.length&& 
			landParcelID.length == borough.length ){
			
			landParcelArray = new LandParcel[landParcelID.length];
			Vector[] householdsInParcel = new Vector[landParcelID.length];
			LandParcel.init(structureTypeArray,householdTypeArray);
			for (int i=0; i<landParcelID.length; i++){
				LandParcel b = new LandParcel(i+1, area[i], numberOfBuildings[i],
						initialStructureType[i],borough[i],boroughArray, buildingPositions,
						proportionToBuy, costs, 
						greenery, (int) this.maximumPropertyDistances[TYPE_GREENERY], attractivenessGreenery[0].length);
				landParcelArray[i] = b;
			} 
			boolean done =false;
			int run=0;
			int hhlast=0;
			for (int i=0; i<allHouseholdsOfBorough.length; i++) {
				hhlast += allHouseholdsOfBorough[i].size();
			}
			while(!done) {
				run++;
				int hh =0;
				done=true;
				for (int i=0; i<allHouseholdsOfBorough.length; i++) {
					hh += allHouseholdsOfBorough[i].size();
					if(allHouseholdsOfBorough[i].size()>0) done=false;
				}
				if(done || (hh==hhlast&&run!=1) ) break;
				hhlast=hh;
				calculateAttractiveness();
				
				extractVacantFlats(true);
				System.out.println("sorting run "+run+";  vacant flats: "+vacantFlats.size()+" hh:"+hh);
				
				for (int i=0; i<allHouseholdsOfBorough.length; i++) {
					extractVacantFlats(boroughID[i]);
					System.out.println("sorting borough "+i+
										";  vacant flats: "+vacantFlats.size()+
										";  housholds: "+allHouseholdsOfBorough[i].size());
					for (int k=0; k<allHouseholdsOfBorough[i].size(); k++) {
						
						// ????  extractVacantFlats();
						
						Household house = ( (Household) allHouseholdsOfBorough[i].elementAt(k));
						//System.out.println("size flats: "+vacantFlats.size());
						float[] cumAttr=new float[vacantFlats.size()];
						for(int v = 0; v < vacantFlats.size(); v++) {
							Flat flat = ( (Flat) vacantFlats.elementAt(v));
							//if( flat.getLandParcel().getBorough()-1 == i ) {
								if(v==0) cumAttr[v]=flat.getAttractiveness()[house.getHouseholdType().getID()-1]*( (Integer)numOfVacantFlats.elementAt(v)).intValue();
								else cumAttr[v]=cumAttr[v-1]+flat.getAttractiveness()[house.getHouseholdType().getID()-1]*( (Integer)numOfVacantFlats.elementAt(v)).intValue();   
								//if(flat.getAttractiveness()[house.getHouseholdType().getID()-1]==0) 
								//System.out.println(flat.getAttractiveness()[house.getHouseholdType().getID()-1]);
							//}
							/*else {
								if(v==0) cumAttr[v]=0;
								else cumAttr[v]=cumAttr[v-1];
							}*/	
						}
						float random = 0;
						if(vacantFlats.size()>0) {
							random = (float) (Math.random()*cumAttr[cumAttr.length-1]);
						}
						Flat flat = null;
						boolean stop=false;
						//System.out.println(cumAttr.length+", last: "+cumAttr[cumAttr.length-1]);
						for(int v = 0; v < cumAttr.length; v++) {
							if(random < cumAttr[v]) {
								//System.out.println("found, "+numOfVacantFlats.elementAt(v));
								flat=vacantFlats.elementAt(v);
								//house.setFlat(flat);
								//landParcelArray[flat.getLandParcel().getID()-1].addHousehold(house, flat);
								
								house.moving(flat);
								house.setInitialLastMoving();
								
								allHouseholdsOfTheCity.addElement(house);
								
								allHouseholdsOfBorough[i].removeElementAt(k);
								
								int numVacant=numOfVacantFlats.elementAt(v);
								numOfVacantFlats.removeElementAt(v);
								numOfVacantFlats.insertElementAt(numVacant-1, v);
								
								k--;
								stop=true;
								break;
							}	
							//if(stop) break;
						}
						//if(stop==false) System.out.println(random+"   "+cumAttr[cumAttr.length-1]);
						//if(stop) break;
					}
					//if(stop) break;	
				}
				if(run%100==0) System.out.println("sorting run "+run+";  households remaining: "+hh);
							
			}
			System.out.println("sorting "+allHouseholdsOfTheCity.size()+" households into flats complete");

		}
		else 
			throw new UnequalInputArrayLengthException("Problems with input arrays for land parcels");
		
		
		int[] hhNew=new int[householdTypeArray.length];
		int[] hhDiff=new int[householdTypeArray.length];
		newHouseholdsOfTheCity = new Vector<Household>();
		
		meanHouseholdSize=new float[householdTypeArray.length];
		float[] nrHH=new float[householdTypeArray.length];
		for(Household h : allHouseholdsOfTheCity) {
			meanHouseholdSize[h.getHouseholdTypeID()-1] += h.getNumPerson();
			nrHH[h.getHouseholdTypeID()-1]++;
		}
		for(Household h : newHouseholdsOfTheCity) {
			meanHouseholdSize[h.getHouseholdTypeID()-1] += h.getNumPerson();
			nrHH[h.getHouseholdTypeID()-1]++;
		}
		for(int i=0;i<meanHouseholdSize.length;i++) {
			meanHouseholdSize[i] /= nrHH[i];
		}
		
		
		
		
		int[] cohorts = countCohorts(allHouseholdsOfTheCity, newHouseholdsOfTheCity);
		float[] expectedHouseholds = calculateExpectedHouseholds(cohorts);
		for(int i=0;i<allHouseholdsOfTheCity.size();i++) {
			hhNew[allHouseholdsOfTheCity.elementAt(i).getHouseholdTypeID()-1]++;
		}
		for(int i=0;i<newHouseholdsOfTheCity.size();i++) {
			hhNew[newHouseholdsOfTheCity.elementAt(i).getHouseholdTypeID()-1]++;
		}
		
		for(int i=0;i<hhNew.length;i++) {
			hhDiff[i]=hhNew[i]-(int)expectedHouseholds[i];
		}
		
		ufz.aloe.urban.controler.UrbanModelControler.printArray(hhDiff);
		
		mergeHouseholds(hhDiff);
		
		calculateHouseholdsMoveIn();
		
		System.out.println("initial:");
		ufz.aloe.urban.controler.UrbanModelControler.printArray(hhNew);
		ufz.aloe.urban.controler.UrbanModelControler.printArray(expectedHouseholds);
		ufz.aloe.urban.controler.UrbanModelControler.printArray(hhDiff);
		ufz.aloe.urban.controler.UrbanModelControler.printArray(meanHouseholdSize);
	}
	public void outputPerTimeStep(int t){
		//Vector<Building> bb=new Vector<Building>();
		try {
			PrintWriter out;
			String step=""+t;
			while(step.length()<3) step="0"+step;
			out = new PrintWriter(new BufferedWriter( 
					new FileWriter ("results/output_step"+step+".txt")));
					
					//berschrift
					
			out.print("BuildingID,x,y,LandParcelID,BoroughID"
					+",structuretype,numFlats,Vacant flats"
					+",HHT1,HHT2,HHT3,HHT4,HHT5,HHT6,HHT7,sum_HHT"
					+",ATT1,ATT2,ATT3,ATT4,ATT5,ATT6,ATT7,sum_ATT");
					
			for(int i=0;i<cohortBirth.length;i++) {
				out.print(",cohort_"+(i));
			}	
			out.print(",sum_INH");
			
			for(int i=0;i<budget.length;i++) {
				out.print(",HHT_BUDGET_"+(i));
			}
				
			out.println();
			
			
			
			PrintWriter outP;
			outP = new PrintWriter(new BufferedWriter( 
					new FileWriter ("results/parcel_output_step"+step+".txt")));
					
			outP.print("LandParcelID,BoroughID"
					+",numFlats,Vacant flats");
			
			
			for(int i=0;i<householdTypeArray.length;i++) {
				outP.print(",HHT_"+(i+1));
			}	
			outP.print(",sum_HH");	
			
			
				
			for(int i=0;i<cohortBirth.length;i++) {
				outP.print(",COH_"+(i+1));
			}	
			outP.print(",sum_COH");	
			
			
			
			
			
			for(int i=0;i<budget.length;i++) {
				outP.print(",HHT_BUDGET_"+(i));
			}
			
			for(int i=0;i<householdTypeArray.length;i++) {
				outP.print(",MIGRATE_"+(i+1));
			}
				
			outP.println();
					
			
			for (int i=0; i<landParcelArray.length; i++){
				Vector<Building> pb=landParcelArray[i].getBuildings();
				
				outP.print(
						landParcelArray[i].getID()+"," +
						landParcelArray[i].getBorough()
					);
				int nFlat=0;
				int nvFlat=0;
				int sumHHTP=0;
				int sumCOHP=0;
				int[] hhtP=new int[householdTypeArray.length];
				int[] cohP=new int[cohortBirth.length];
				int[] hht_budget=new int[budget.length];
				
				for (Building b : pb) {
					
					int[] b_hht_budget=new int[budget.length];
					
					nFlat += b.getNumberOfFlats();
					nvFlat += b.getVacancy();
					out.print(
						b.getID()+"," +
						b.getX()+"," +
						b.getY()+"," +
						b.getLandParcelID()+"," +
						b.getBoroughID()+"," +
						b.getStructureTypeID()+"," +
						b.getNumberOfFlats()+"," +
						b.getVacancy()
					);
					int[] hht=b.getHouseholdTypes(householdTypeArray);
					int sumHH=0;
					for(int j=0;j<hht.length;j++) {
						out.print(","+hht[j]);
						hhtP[j]+=hht[j];
						sumHHTP += hht[j];
						sumHH += hht[j];	
					}
					out.print(","+sumHH);
					
					float[] att=b.getAttractiveness();
					float sumATT=0;
					for(int j=0;j<att.length;j++) {
						out.print(","+att[j]);
						sumATT += att[j];	
					}
					out.print(","+sumATT);
					
					int[] coh=b.getCohorts(new int[cohortBirth.length]);
					int sumCOH=0;
					for(int j=0;j<coh.length;j++) {
						out.print(","+coh[j]);	
						cohP[j]+=coh[j];
						sumCOHP += coh[j];	
						sumCOH += coh[j];	
					}
					out.print(","+sumCOH);
					
					Vector<Household> hh=b.getHouseholds();
					for (Household h : hh) {
						hht_budget[h.getBudget()]++;
						b_hht_budget[h.getBudget()]++;
						
					}
					
					for(int j=0;j<b_hht_budget.length;j++) {
						out.print(","+b_hht_budget[j]);	
					}
					
					out.println("");
					
				}
				
				outP.print(","+nFlat);
				outP.print(","+nvFlat);
				
				for(int j=0;j<hhtP.length;j++) {
					outP.print(","+hhtP[j]);
				}
				outP.print(","+sumHHTP);
				
				for(int j=0;j<cohP.length;j++) {
					outP.print(","+cohP[j]);
				}
				outP.print(","+sumCOHP);
				
				
				
				
				
				for(int j=0;j<budget.length;j++) {
					outP.print(","+hht_budget[j]);
				}
				
				int[] mig=landParcelArray[i].getMigration();
				
				for(int j=0;j<mig.length;j++) {
					outP.print(","+mig[j]);
				}
				
				
				outP.println("");
				
				
				
				
				
			}
			
			
			
			out.close();
			outP.close();
			
			
			if(movingLastStep!=null) {
				PrintWriter outMove;
				outMove = new PrintWriter(new BufferedWriter( 
					new FileWriter ("results/move_output_step"+step+".txt")));
					
				
				outMove.println(Move.head());
				for(Move m : movingLastStep) {
					outMove.println(m.toString());
				}
				outMove.close();
			}
			
			
			
			if(householdTypeChanges!=null) {
				PrintWriter outMature;
				outMature = new PrintWriter(new BufferedWriter( 
					new FileWriter ("results/mature_output_step"+step+".txt")));
					
				
				for(int k=0;k<householdTypeChanges.length;k++) {
					String s="";
					for(int j=0;j<householdTypeChanges[0].length;j++) {
						s += householdTypeChanges[k][j] + ((j==householdTypeChanges[0].length-1) ? "" : ",");
					}
					outMature.println(s);
				}
				outMature.close();
			}
			
			
				
			
		} catch (FileNotFoundException fnf) {
			System.err.println("File attractivenessOutput.csv not found! Please check if you forgot to " +
					"close this file. "+fnf);
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	/**
	 * this function is only used to check that we have got the input in the right fields
	 */
	public void inputOutput(){	
		try {
			PrintWriter out;
			out = new PrintWriter(new BufferedWriter( 
					new FileWriter ("results/inputOutput.csv")));
		//land parcel
			out.println("LandParcelID"+";"+"area"
					+";"+"initial structure type"+";"+
					"max number of places to occupy"+";"+"borough");//berschrift
			for (int i=0; i< (landParcelArray.length); i++){	
				out.println(landParcelArray[i].getID()+";" +
						landParcelArray[i].getArea()+";" +
						landParcelArray[i].getInitialStructureType()+";" +
						landParcelArray[i].getMaxNumberPlaces2Occupy()+";" +
						landParcelArray[i].getBorough());	
			}
		//household types	
			out.println("householdTypes"+";"+"personsPerHousehold"+";"+
					"attractiveness weights");//berschrift
			for (int i=0; i< (householdTypeArray.length); i++){	
				out.print(householdTypeArray[i].getHouseholdTypes()+";" +
						householdTypeArray[i].getNumPersons()+";");
				for (int j=0; j< (householdTypeArray.length); j++){	
					out.print(householdTypeArray[i].getAttractivenessWeightsPopDens()[j]+";");
				}
				out.println();
			}
		//boroughs	
			out.println("boroughID"+";"+"percentVacancy"+";"+
			"percent householdType");//berschrift
		
			for (int i=0; i< (boroughArray.length); i++){	
				out.print(boroughArray[i].getBoroughID()+";" +
						boroughArray[i].getPercentVacancy()+";");
				for (int j=0; j< (cohortBirth.length); j++){	
					out.print(boroughArray[i].getPercentCohort()[j]+";");
				}
				out.println();
		
			}
			out.close();
		
		} catch (FileNotFoundException fnf) {
			System.err.println("File attractivenessOutput.csv not found! Please check if you forgot to " +
					"close this file. "+fnf);
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	
	public void calculateAttractiveness (){
		for(int j=0;j<landParcelArray.length;j++){
				//landParcelArray[j].calculateInhabitantsPerArea(householdTypeArray, structureTypeArray);
				landParcelArray[j].calculateAttractiveness(householdTypeArray, 
															landParcelArray,
															landParcelDistances,
															infrastructure,
															maximumPropertyDistances,
															weightsOfParameters);
		}
		setChanged();  
		
	    notifyObservers  (landParcelArray);
	    

	}
	
	
	public int[] countCohorts(Vector<Household> allHouseholdsOfTheCity, Vector<Household> newHouseholdsOfTheCity){
		int[] cohorts=new int[cohortBirth.length];
		for(int i=0;i<allHouseholdsOfTheCity.size();i++) {
			Household h=allHouseholdsOfTheCity.elementAt(i);
			Vector<Inhabitant> inh = h.getInhabitants();
			for(int j=0;j<inh.size();j++) {
				cohorts[inh.elementAt(j).getCohort()]++;
			}
		}
		if(newHouseholdsOfTheCity!=null) {
			for(int i=0;i<newHouseholdsOfTheCity.size();i++) {
				Household h=newHouseholdsOfTheCity.elementAt(i);
				Vector<Inhabitant> inh = h.getInhabitants();
				for(int j=0;j<inh.size();j++) {
					cohorts[inh.elementAt(j).getCohort()]++;
				}
			}
		}
		return cohorts;
	}
	public int[] getCohorts(){
		return countCohorts(allHouseholdsOfTheCity, newHouseholdsOfTheCity);
	}
	public float[] calculateExpectedHouseholds(int[] cohorts){
		float[] expectedHouseholds=new float[householdTypeArray.length];

		for(int i=0;i<expectedHouseholds.length;i++) {
			for(int j=0;j<cohortBirth.length;j++) {
				expectedHouseholds[i] += 
					(cohorts[j]*cohortToHousehold[j][i])
					///(meanHouseholdSize[i]);
					/ ((HouseholdType.getPersonPerHousehold()[i]+HouseholdType.getMaxPersonPerHousehold()[i])/2);
			}
		}
		return expectedHouseholds;
	}

	
	public static int [] getIntArray (int lineNr, Vector resultVec){
		// reads the lineNr line from the Vector resultVec into an array of integers
	
		Vector tmpVec;
		tmpVec = (Vector) resultVec.elementAt(lineNr);
		int a []= new int [tmpVec.size()-1];
		
		for(int j = 0; j < (tmpVec.size()-1);j++){
			a[j] = Type.toInt(tmpVec.elementAt(j+1).toString()); 
		}
		return a;
	}
	public static int [] [] getIntMatrix (int lineNr, Vector resultVec){
		// reads all lines from lineNr downwards into a matrix of floats
	
		Vector tmpVec;
		tmpVec = (Vector) resultVec.elementAt(lineNr);
		int a [][]= new int [tmpVec.size()-1][resultVec.size()-lineNr];
		
		for(int i = 0; i < (resultVec.size()-lineNr);i++){
			tmpVec = (Vector) resultVec.elementAt(lineNr+i);
			for(int j = 0; j < (tmpVec.size()-1);j++){
			a[j][i] = Type.toInt(tmpVec.elementAt(j+1).toString()); 
			//System.out.println(j+"   "+i+"       "+tmpVec.elementAt(j+1).toString());
			}
		}
		return a;
	}
	public static String [] getStringArray (int lineNr, Vector resultVec){
		// reads the lineNr line from the Vector resultVec into an array of integers
	
		Vector tmpVec;
		tmpVec = (Vector) resultVec.elementAt(lineNr);
		String a []= new String [tmpVec.size()-1];
		

		for(int j = 0; j < (tmpVec.size()-1);j++){
			a[j] = (tmpVec.elementAt(j+1)).toString(); 
					}
		return a;
	}

	public static float [] getFloatArray (int lineNr, Vector resultVec){
		// reads the lineNr line from the Vector resultVec into an array of integers
	
		Vector tmpVec;
		tmpVec = (Vector) resultVec.elementAt(lineNr);
		float a []= new float [tmpVec.size()-1];
		

		for(int j = 0; j < (tmpVec.size()-1);j++){
			a[j] = Type.toFloat(tmpVec.elementAt(j+1).toString());
					}
		return a;
	}

	public static float [] [] getFloatMatrix (int lineNr, Vector resultVec){
		// reads all lines from lineNr downwards into a matrix of floats
	
		Vector tmpVec;
		tmpVec = (Vector) resultVec.elementAt(lineNr);
		float a [][]= new float [tmpVec.size()-1][resultVec.size()-lineNr];
		
		for(int i = 0; i < (resultVec.size()-lineNr);i++){
			tmpVec = (Vector) resultVec.elementAt(lineNr+i);
			for(int j = 0; j < (tmpVec.size()-1);j++){
			a[j][i] = Type.toFloat(tmpVec.elementAt(j+1).toString()); 
			//System.out.println(j+"   "+i+"       "+tmpVec.elementAt(j+1).toString());
			}
		}
		return a;
	}
	

	/**
	 * @return Returns the boroughArray.
	 */
	public Borough[] getBoroughArray() {
		return boroughArray;
	}


	/**
	 * @return Returns the householdTypeArray.
	 */
	public HouseholdType[] getHouseholdTypeArray() {
		return householdTypeArray;
	}


	/**
	 * @return Returns the landParcelArray.
	 */
	public LandParcel[] getLandParcelArray() {
		return landParcelArray;
	}
	/**
	 * extracts all vacant flats from the land parcels.
	 * the extracted flats are writen to vacantFlats, numOfVacantFlats indicates the number of each 
	 * flat type (determined by land parcel and structure type
	 */
	public void extractVacantFlats(boolean random) {
		vacantFlats = new Vector<Flat>();
		numOfVacantFlats = new Vector<Integer>();
		for(int i = 0; i < landParcelArray.length; i++) {
			Vector<Building> builds=landParcelArray[i].getBuildings();
			//System.out.println(i+":  "+builds.size());
			for(Building b : builds) {
				if(b.getVacancy()>0 && !b.remove(minYearsVacant)) {
					vacantFlats.add(b.getFlat());
					numOfVacantFlats.add( b.getVacancy() );
				}
			}
		}
		if (random) {
			Vector<Flat> rFlats=new Vector<Flat>();
			Vector<Integer> rNumFlats=new Vector<Integer>();
			while(vacantFlats.size()>0) {
				int r=(int) (Math.random()*vacantFlats.size());
				rFlats.add(vacantFlats.elementAt(r));
				vacantFlats.removeElementAt(r);
				rNumFlats.add(numOfVacantFlats.elementAt(r));
				numOfVacantFlats.removeElementAt(r);
			}
			vacantFlats = rFlats;
			numOfVacantFlats = rNumFlats;
		}
		
	}
	public void extractVacantFlats(int boroughID) {
		vacantFlats = new Vector<Flat>();
		numOfVacantFlats = new Vector<Integer>();
		for(int i = 0; i < landParcelArray.length; i++) {
			if( landParcelArray[i].getBorough() == boroughID ) {
				Vector<Building> builds=landParcelArray[i].getBuildings();
				//System.out.println(i+":  "+builds.size());
				for(Building b : builds) {
					if(b.getVacancy()>0) {
						vacantFlats.add(b.getFlat());
						numOfVacantFlats.add( b.getVacancy() );
					}
				}
			}
		}
		
	}
	public void matureHouseholds() {
		
		System.out.println("starting matureHouseholds;     "+allHouseholdsOfTheCity.size()+" / "+newHouseholdsOfTheCity.size());
		
		householdTypeChanges=new int[householdTypeArray.length][householdTypeArray.length];
		
		int[] hhDiff=new int[householdTypeArray.length];
		//System.out.println("Households with flat: "+allHouseholdsOfTheCity.size());
		//System.out.println("Households without flat: "+newHouseholdsOfTheCity.size());
		allHouseholdsOfTheCity.addAll(newHouseholdsOfTheCity);
		newHouseholdsOfTheCity = new Vector<Household>();
		for(int i=0;i<allHouseholdsOfTheCity.size();i++) {
			if(allHouseholdsOfTheCity.elementAt(i).getHouseholdType()==null) {
				allHouseholdsOfTheCity.elementAt(i).removeFromModel();
				allHouseholdsOfTheCity.removeElementAt(i);
				
				i--;
				break;
			}
			else {
				if(allHouseholdsOfTheCity.elementAt(i).getFlat()==null) {
					newHouseholdsOfTheCity.addElement(allHouseholdsOfTheCity.elementAt(i));
					allHouseholdsOfTheCity.removeElementAt(i);
				
					i--;
					break;
				}
			}
			if(allHouseholdsOfTheCity.elementAt(i).getHouseholdType()==null) {
				allHouseholdsOfTheCity.elementAt(i).removeFromModel();
				allHouseholdsOfTheCity.removeElementAt(i);
				i--;
				break;
			}
			int hht=allHouseholdsOfTheCity.elementAt(i).getHouseholdTypeID()-1;
			Vector<Household> h2=allHouseholdsOfTheCity.elementAt(i).mature();
			
			if(h2!=null && h2.size()!=0) {
				if(allHouseholdsOfTheCity.elementAt(i).getHouseholdType()!=null) {
					householdTypeChanges[hht][allHouseholdsOfTheCity.elementAt(i).getHouseholdTypeID()-1]++; 
				}
				for(Household h : h2) {
					householdTypeChanges[hht][h.getHouseholdTypeID()-1]++; 	
				}
				
				newHouseholdsOfTheCity.addAll(h2);
			}
			if(allHouseholdsOfTheCity.elementAt(i).getHouseholdType()==null) {
				allHouseholdsOfTheCity.elementAt(i).removeFromModel();
				allHouseholdsOfTheCity.removeElementAt(i);
				i--;
				break;
			}
		}
		allHouseholdsOfTheCity.removeAll(newHouseholdsOfTheCity);
		for(int i=0;i<newHouseholdsOfTheCity.size();i++) {
			if(newHouseholdsOfTheCity.elementAt(i).getHouseholdType()==null) {
				newHouseholdsOfTheCity.removeElementAt(i);
				i--;
			}
		}
		
		System.out.println("matureHouseholds, null HH removed;     "+allHouseholdsOfTheCity.size()+" / "+newHouseholdsOfTheCity.size());
	
		
		int[] cohorts = countCohorts(allHouseholdsOfTheCity, newHouseholdsOfTheCity);
		float[] expectedHouseholds = calculateExpectedHouseholds(cohorts);
		//countHouseholds();
		int[] hhNew=countHouseholds();
		
		
		for(int i=0;i<hhNew.length;i++) {
			hhDiff[i]=hhNew[i]-(int)expectedHouseholds[i];
		}
		
		//System.out.println("");
		//System.out.print("before merging:  ");
		//ufz.aloe.urban.controler.UrbanModelControler.printArray(hhDiff);
		
		mergeHouseholds(hhDiff);
		
		//ufz.aloe.urban.controler.UrbanModelControler.printArray(hhNew);
		//ufz.aloe.urban.controler.UrbanModelControler.printArray(expectedHouseholds);
		//System.out.print("after merging:  ");
		//ufz.aloe.urban.controler.UrbanModelControler.printArray(hhDiff);
		
		System.out.println("matureHouseholds done;     "+allHouseholdsOfTheCity.size()+" / "+newHouseholdsOfTheCity.size());
	
		
		//ufz.aloe.urban.controler.UrbanModelControler.printArray(meanHouseholdSize);
	}
	
	public void mergeHouseholds(int[] hhDiff) {
		
		System.out.println("starting mergeHouseholds;     "+allHouseholdsOfTheCity.size()+" / "+newHouseholdsOfTheCity.size());
	
		if(householdTypeChanges==null) householdTypeChanges=new int[householdTypeArray.length][householdTypeArray.length];
		
		Vector<Household>[] hh=new Vector[householdTypeArray.length];
		for(int i=0;i<householdTypeArray.length;i++) hh[i]=new Vector<Household>();
		
		for(int i=0;i<allHouseholdsOfTheCity.size();i++) {
			if(allHouseholdsOfTheCity.elementAt(i).getHouseholdType()==null) {
				allHouseholdsOfTheCity.elementAt(i).removeFromModel();
				allHouseholdsOfTheCity.removeElementAt(i);
				i--;
			}
			else hh[allHouseholdsOfTheCity.elementAt(i).getHouseholdTypeID()-1].add(allHouseholdsOfTheCity.elementAt(i));
		}
		for(int i=0;i<newHouseholdsOfTheCity.size();i++) {
			hh[newHouseholdsOfTheCity.elementAt(i).getHouseholdTypeID()-1].add(newHouseholdsOfTheCity.elementAt(i));
		}
		
		this.allHouseholdsOfTheCity=new Vector<Household>();
		this.newHouseholdsOfTheCity=new Vector<Household>();
		
		/*float sumPred=0;
		for(int i=0;i<expectedHouseholds.length;i++) {
			sumPred += expectedHouseholds[i];
		}*/
		while (true) {
			boolean done=true;
			/*float sumCurr=0;
			for(int i=0;i<hh.length;i++) {
				sumCurr += hh[i].size();
			}*/
			// too many young singles
			//if(hhDiff[0]>0) {
			if(hhDiff[0]>0) {
				// too many single parents, not enough families
				if(hhDiff[5]>0 && hhDiff[4]<0) {
					Household hS=hh[0].elementAt(random.nextInt(hh[0].size()));
					hh[0].remove(hS);
					hS.removeFromModel();
					hhDiff[0]--;
					
					Household hSp=hh[5].elementAt(random.nextInt(hh[5].size()));
					hh[5].remove(hSp);
					hSp.removeFromModel();
					hhDiff[5]--;
					
					Household hNew = new Household(1, householdTypeArray[4]);
					hhDiff[4]++;
					
					
					householdTypeChanges[0][4]++;
					householdTypeChanges[5][4]++; 
								
					for(Inhabitant i : hS.getInhabitants()) {
						hNew.addPersonUnchecked(i);
					}
					for(Inhabitant i : hSp.getInhabitants()) {
						hNew.addPersonUnchecked(i);
					}
					newHouseholdsOfTheCity.add(hNew);
					done=false;
				}
				// not enough young cohabitation
				else if(hhDiff[2]<0) {
					Household hS1=hh[0].elementAt(random.nextInt(hh[0].size()));
					hh[0].remove(hS1);
					hS1.removeFromModel();
					hhDiff[0]--;
					
					Household hS2=hh[0].elementAt(random.nextInt(hh[0].size()));
					hh[0].remove(hS2);
					hS2.removeFromModel();
					hhDiff[0]--;
					
					Household hNew = new Household(1, householdTypeArray[2]);
					hhDiff[2]++;
					
					
					householdTypeChanges[0][2] += 2;
					
					
					for(Inhabitant i : hS1.getInhabitants()) {
						hNew.addPersonUnchecked(i);
					}
					for(Inhabitant i : hS2.getInhabitants()) {
						hNew.addPersonUnchecked(i);
					}
					newHouseholdsOfTheCity.add(hNew);
					done=false;
				}
				// not enought flat sharers
				else if(hhDiff[6]<0) {
					Household hS1=hh[0].elementAt(random.nextInt(hh[0].size()));
					hh[0].remove(hS1);
					hS1.removeFromModel();
					hhDiff[0]--;
					
					Household hS2=hh[0].elementAt(random.nextInt(hh[0].size()));
					hh[0].remove(hS2);
					hS2.removeFromModel();
					hhDiff[0]--;
					
					Household hS3=hh[0].elementAt(random.nextInt(hh[0].size()));
					hh[0].remove(hS3);
					hS3.removeFromModel();
					hhDiff[0]--;
					
					Household hNew = new Household(1, householdTypeArray[6]);
					hhDiff[6]++;
					
					householdTypeChanges[0][6] += 3;
					
					for(Inhabitant i : hS1.getInhabitants()) {
						hNew.addPersonUnchecked(i);
					}
					for(Inhabitant i : hS2.getInhabitants()) {
						hNew.addPersonUnchecked(i);
					}
					for(Inhabitant i : hS3.getInhabitants()) {
						hNew.addPersonUnchecked(i);
					}
					newHouseholdsOfTheCity.add(hNew);
					done=false;
				}
			}
			// too many old singles
			if(hhDiff[1]>0) {
				// not enough old cohabitation
				if(hhDiff[3]<0) {
					Household hS1=hh[1].elementAt(random.nextInt(hh[1].size()));
					hh[1].remove(hS1);
					hS1.removeFromModel();
					hhDiff[1]--;
					
					Household hS2=hh[1].elementAt(random.nextInt(hh[1].size()));
					hh[1].remove(hS2);
					hS2.removeFromModel();
					hhDiff[1]--;
					
					Household hNew = new Household(1, householdTypeArray[3]);
					hhDiff[3]++;
					
					
					householdTypeChanges[1][3] += 2;
					
					
					for(Inhabitant i : hS1.getInhabitants()) {
						hNew.addPersonUnchecked(i);
					}
					for(Inhabitant i : hS2.getInhabitants()) {
						hNew.addPersonUnchecked(i);
					}
					newHouseholdsOfTheCity.add(hNew);
					done=false;
				}
			}
			// too many old singles
			if(done && hhDiff[1]>0) {
				// too many single parents, not enough families
				if(hhDiff[5]>0 && hhDiff[4]<0) {
					Household hS=hh[1].elementAt(random.nextInt(hh[1].size()));
					hh[1].remove(hS);
					hS.removeFromModel();
					hhDiff[1]--;
					
					Household hSp=hh[5].elementAt(random.nextInt(hh[5].size()));
					hh[5].remove(hSp);
					hSp.removeFromModel();
					hhDiff[5]--;
					
					Household hNew = new Household(1, householdTypeArray[4]);
					hhDiff[4]++;
					
					
					householdTypeChanges[1][4]++;
					householdTypeChanges[5][4]++;
					
					
					for(Inhabitant i : hS.getInhabitants()) {
						hNew.addPersonUnchecked(i);
					}
					for(Inhabitant i : hSp.getInhabitants()) {
						hNew.addPersonUnchecked(i);
					}
					newHouseholdsOfTheCity.add(hNew);
					done=false;
				}
			}
			// too many young cohabitation
			if(done && hhDiff[2]>0) {
				if(hhDiff[0]<0 
					|| hhDiff[6]<0 
					|| (hhDiff[5]>0 && hhDiff[4]<0)) {
						
					Household hC=hh[2].elementAt(random.nextInt(hh[2].size()));
					hh[2].remove(hC);
					hC.removeFromModel();
					hhDiff[2]--;
					
					householdTypeChanges[2][0] += 2;

					for(Inhabitant i : hC.getInhabitants()) {
						Household hNew = new Household(1, householdTypeArray[0]);
						hNew.addPersonUnchecked(i);
						hhDiff[0]++;
						newHouseholdsOfTheCity.add(hNew);
					}
					done=false;						
				}
			}
			if(done) break;
		}
		
		//this.allHouseholdsOfTheCity=new Vector<Household>();
		//this.newHouseholdsOfTheCity=new Vector<Household>();
		
		System.out.println("mergeHouseholds done, sorting to all/new;     "+allHouseholdsOfTheCity.size()+" / "+newHouseholdsOfTheCity.size());
			
		for(int i=0;i<householdTypeArray.length;i++) {
			for(int j=0;j<hh[i].size();j++) {
				Household h=hh[i].elementAt(j);
				if(h.getFlat()==null) newHouseholdsOfTheCity.add(h);
				else allHouseholdsOfTheCity.add(h);
			}
		}
		
		System.out.println("mergeHouseholds done;     "+allHouseholdsOfTheCity.size()+" / "+newHouseholdsOfTheCity.size());
	
	}
	
	public int[] countHouseholds() {
		int[] hhNew=new int[householdTypeArray.length]; 
		for(int i=0;i<allHouseholdsOfTheCity.size();i++) {
			if(allHouseholdsOfTheCity.elementAt(i).getHouseholdType()==null) {
				allHouseholdsOfTheCity.elementAt(i).removeFromModel();
				allHouseholdsOfTheCity.removeElementAt(i);
				
				i--;
				break;
			}
			else {
				hhNew[allHouseholdsOfTheCity.elementAt(i).getHouseholdTypeID()-1]++;
			}
			
		}
		for(int i=0;i<newHouseholdsOfTheCity.size();i++) {
			hhNew[newHouseholdsOfTheCity.elementAt(i).getHouseholdTypeID()-1]++;
		}
		return hhNew;
	}
	
	public void calculateHouseholdsMoveIn() {
		Vector<Inhabitant> inhabitants=createInhabitants( percentCohortMoveIn, numMovingIn );
		householdsMoveIn=sortToHouseholds(inhabitants, cohortToHousehold);
	}
	
	public void externalMoveHouseholds() {
		
		System.out.println("starting externalMoveHouseholds;     "+allHouseholdsOfTheCity.size()+" / "+newHouseholdsOfTheCity.size());
	
		
		Vector<Household>[] hh=new Vector[householdTypeArray.length];
		for(int i=0;i<householdTypeArray.length;i++) hh[i]=new Vector<Household>();
		
		for(int i=0;i<allHouseholdsOfTheCity.size();i++) {
			hh[allHouseholdsOfTheCity.elementAt(i).getHouseholdTypeID()-1].add(allHouseholdsOfTheCity.elementAt(i));
		}
		for(int i=0;i<newHouseholdsOfTheCity.size();i++) {
			hh[newHouseholdsOfTheCity.elementAt(i).getHouseholdTypeID()-1].add(newHouseholdsOfTheCity.elementAt(i));
		}
		
		int[] cohOut=new int[percentCohortMoveOut.length];
		for(int i=0;i<percentCohortMoveOut.length;i++) {
			cohOut[i]=(int)(0.01*percentCohortMoveOut[i]*numMovingOut);
		}
			
		float[] expectedHhOut = calculateExpectedHouseholds(cohOut);
		
		for(int i=0;i<householdTypeArray.length;i++){
			while(expectedHhOut[i]>0 && hh[i].size()>0) {
				int n=(int) (Math.random()*hh[i].size());
				hh[i].elementAt(n).removeFromModel();
				hh[i].removeElementAt(n);
				
				expectedHhOut[i]--;
			}
		}
		
		
		this.allHouseholdsOfTheCity=new Vector<Household>();
		this.newHouseholdsOfTheCity=new Vector<Household>();
		
		for(int i=0;i<householdTypeArray.length;i++) {
			for(int j=0;j<hh[i].size();j++) {
				Household h=hh[i].elementAt(j);
				if(h.getFlat()==null) newHouseholdsOfTheCity.add(h);
				else allHouseholdsOfTheCity.add(h);
			}
		}
		
		System.out.println("externalMoveHouseholds, move out done;     "+allHouseholdsOfTheCity.size()+" / "+newHouseholdsOfTheCity.size());
	
		
		//Vector<Inhabitant> inhabitants=createInhabitants( percentCohortMoveIn, numMovingIn );
		//Vector<Household> households=sortToHouseholds(inhabitants, cohortToHousehold);
		Vector<Household> households=new Vector<Household>();
		System.out.println("Moving in: "+householdsMoveIn.size());
		for(Household h : householdsMoveIn) {
			households.add(h.copy());
		}
		newHouseholdsOfTheCity.addAll(households);
		System.out.println("externalMoveHouseholds, move in done;     "+allHouseholdsOfTheCity.size()+" / "+newHouseholdsOfTheCity.size());
	}
	
	
	
	public void moveHouseholds() {
		
		System.out.println("start moving HH;     "+allHouseholdsOfTheCity.size()+" / "+newHouseholdsOfTheCity.size());
		
		movingLastStep=new Vector<Move>();
		
		Vector<Household> householdsMoving = new Vector<Household>();
		
		int moveP=0;
		int move=0;
		
		for(Household h : allHouseholdsOfTheCity) {
			h.updateLastMoving();
			if( h.movingPossible() || h.getFlat()==null || h.getFlat().getBuilding().remove(minYearsVacant) ) {
				int n=(int) (Math.random()*householdsMoving.size());
				householdsMoving.insertElementAt(h, n);
				moveP++;
			}
		}
		System.out.println("move possible:  "+moveP);
		
		for(Household h : newHouseholdsOfTheCity) {
			int n=(int) (Math.random()*householdsMoving.size());
			householdsMoving.insertElementAt(h, n);
		}
		
		System.out.println("move possible2: "+householdsMoving.size());
		
		allHouseholdsOfTheCity.removeAll(householdsMoving);
		newHouseholdsOfTheCity=new Vector<Household>();

		extractVacantFlats(true);
		
		
		// version 1: hh chooses 3 flats randomly, best is chosen, if better than old flat
		
		while(householdsMoving.size()>0) {
			Vector<Household> remove=new Vector<Household>();
			for(Household h : householdsMoving) {
				
				if( vacantFlats.size()==0 ) break;
				
				Vector<Integer> flats=new Vector<Integer>();
				int i=0;
				int n=0;
				while(i<3 && n<20) {
					int random = (int) (Math.random()*vacantFlats.size());
					Flat flat=vacantFlats.elementAt(random);
					int num=numOfVacantFlats.elementAt(random);
					if( num > 0 && flat.getCost()<=h.getBudget()) {
						flats.add(random);
						i++;
					}
					n++;
				}
				int flat=-1;
				float oldAtt = 0;
				if(h.getFlat()!=null && !h.getFlat().getBuilding().remove(minYearsVacant)) {
					oldAtt = h.getFlat().getAttractiveness()[h.getHouseholdType().getID()-1];
				}
				float maxAtt=0;		
				for(int f : flats) {
					float newAtt = vacantFlats.elementAt(f).getAttractiveness()[h.getHouseholdType().getID()-1];
					if(newAtt>maxAtt 
							&& (oldAtt==0 || newAtt-oldAtt > thresholdMoving[h.getHouseholdType().getID()-1]) )	{
						maxAtt=newAtt;
						flat=f;
					}
				}
				if(flat>=0) {
					Flat fl=vacantFlats.elementAt(flat); 
					int num=numOfVacantFlats.elementAt(flat);
					Flat old=h.getFlat();
					boolean count=false;
					if(h.getFlat()!=null && h.getFlat().getBuilding().remove(minYearsVacant)) count=true;
					if(h.moving(vacantFlats.elementAt(flat))){
						if(count) move++;
						remove.add(h);
						movingLastStep.add( new Move(h, old, vacantFlats.elementAt(flat)) );
						
						if( num <= 1 ) {
							vacantFlats.removeElementAt(flat);
							numOfVacantFlats.removeElementAt(flat);
						}
						else {
							numOfVacantFlats.setElementAt(num-1, flat);	
						}	
					}
					
					
				}
				else {
					if(h.getFlat()!=null && !h.getFlat().getBuilding().remove(minYearsVacant)) {
						remove.add(h);
					}
				}
			}
			allHouseholdsOfTheCity.addAll(remove);
			householdsMoving.removeAll(remove);
		}
		
		System.out.println("move: "+move);
		
		

		System.out.println("moveHouseholds;       nr. HH: "+(newHouseholdsOfTheCity.size()+allHouseholdsOfTheCity.size()));
		
	}


	public Vector<Point> findPlacesToBuildUp(int n, float minRadius) {
		
		Vector<Point> places=new Vector<Point>();
		
		while(places.size()<n) {
			double x=Math.random();
			double y=Math.random();
			
			double xl=landParcelId.xll();
			double xr=landParcelId.xll()+landParcelId.cellsize()*landParcelId.sizeX();
			double yt=landParcelId.yul();
			double yb=landParcelId.yll();
			
			x= xl + (x*(xr-xl));
			y= yb + (y*(yt-yb));
			
			Point pos=new Point((int)x,(int)y);
			
			int[] coordsRel=landParcelId.getCoordsRelative((float)x,(float)y);
			int parcel=(int)landParcelId.valueAt(coordsRel[0], coordsRel[1]);
			
			boolean buildingPossible=true;
			
			for (int j =0; j < landParcelArray.length; j++){
				if(landParcelDistances[parcel][landParcelArray[j].getID()] 
						<= minRadius*2) {
					{
						Vector<Building> bb=landParcelArray[j].getBuildings();
						for (Building b : bb){
							if(b.distance(pos)<=minRadius+b.getStructureType().getMeanRadius()) {
								buildingPossible=false;
								break;
							}
						}
						
					}
				}
			}
			for (Point p : places){
				if(p.distance(pos)<=minRadius*2) {
					buildingPossible=false;
					break;
				}
			}
			if(buildingPossible) {
				places.add(pos);
			}
		
		
		}
		return places;
		
	}

	public void findBuildingsToRemove() {
		buildingsToRemove=new Vector<Building>();
		for(int i = 0; i < landParcelArray.length; i++) {
			Vector<Building> builds=landParcelArray[i].getBuildings();
			for(Building b : builds) {
				b.updateNextStep(0.8f);
				if( b.remove(minYearsVacant) ) {
					buildingsToRemove.add(b);
				}
			}
		}
		
	}
	
	public void removeBuildings() {
		for(Building b : buildingsToRemove) {
			if(b.getPercentVacancy()==1) {
				landParcelArray[b.getLandParcelID()-1].getBuildings().remove(b);
			}
		}
	}

	/**
	 * @return Returns the structureTypeArray.
	 */
	public StructureType[] getStructureTypeArray() {
		return structureTypeArray;
	}
	
	public float[][] getLandParcelDistances() {
		return landParcelDistances;
	}
	public float getLandParcelDistance(int from, int to) {
		return landParcelDistances[from][to];
	}
	public float[] getMaximumPropertyDistances() {
		return maximumPropertyDistances;
	}
	public float getMaximumPropertyDistances(int property) {
		return maximumPropertyDistances[property];
	}
	public Vector<Inhabitant> createInhabitants(float[] percentCoh, int inhab) {
		Vector<Inhabitant> inhabitants = new Vector<Inhabitant>();
		float[] cumPercentCohort = new float[percentCoh.length];
		for(int coh = 0; coh < percentCoh.length; coh++) {
			if(coh==0) cumPercentCohort[coh] = percentCoh[coh]/100;
			else cumPercentCohort[coh] = cumPercentCohort[coh-1]+percentCoh[coh]/100;
		}
		for(int inh = 0; inh < inhab; inh++) {
			float ran = (float) Math.random();
			int c=0;
			for(int coh = 0; coh < cumPercentCohort.length; coh++) {
				if(ran <= cumPercentCohort[coh] && ran > ((coh==0) ? 0 : cumPercentCohort[coh-1]) ) {
					c=coh;
					break;
				}
			}
			
			inhabitants.addElement(new Inhabitant(c, (int)(Math.random()*5) ) );
		
		}
		return inhabitants;
			
	}
	public Vector<Household> sortToHouseholds(Vector<Inhabitant> inhabs, float[][] cohortToHouseh) {
		Vector<Household> households=new Vector<Household>();
		float[][] cumCohortToHouseh = new float[cohortToHouseh.length][cohortToHouseh[0].length];
		for(int i = 0; i < cumCohortToHouseh.length; i++) {
			for(int j = 0; j < cumCohortToHouseh[0].length; j++) {
				if(j==0) cumCohortToHouseh[i][j] = cohortToHouseh[i][j];
				else cumCohortToHouseh[i][j] = cumCohortToHouseh[i][j-1]+cohortToHouseh[i][j];
			}
		}
		boolean done = false;
		boolean progress=false;
		int noProgress=0;
		int run = 0;
		while(!done && noProgress<=10) {
			int[] hhincomplete=new int[householdTypeArray.length];
			int[] hhcomplete=new int[householdTypeArray.length];
			progress=false;
			int hhcomp=0,
				hhincomp=0,
				pershh=0,
				persnohh=0;
			System.out.println("run "+run);
			run++;
			done=true;
			
			int incomplete=0;
			
			for (int j=0; j<inhabs.size(); j++) {
				done=false;
				float ran = (float) Math.random();
				int h = 0;
				Inhabitant inhab = (Inhabitant) inhabs.elementAt(j);
				for(int hht = 0; hht < cumCohortToHouseh.length; hht++) {
					if(ran <= cumCohortToHouseh[ inhab.getCohort()][hht] && ran > ((hht==0) ? 0 : cumCohortToHouseh[ inhab.getCohort()][hht-1]) ) {
						h=hht;
						break;
					}
				}
				boolean newHousehold = true;
				for(int hhb=0; hhb<households.size(); hhb++) {
					Household hCurr = (Household) households.elementAt(hhb);
					if( (noProgress>=10 || hCurr.getHouseholdTypeID() == h+1) && hCurr.addPerson(inhab)) {
						newHousehold = false;
						inhabs.removeElementAt(j);
						j--;
						progress=true;
						break;
					}
				}
				if(newHousehold && Math.random() < 1) {
					Household hCurr = new Household(1, householdTypeArray[h]);
					if(hCurr.addPerson(inhab)) {
						households.addElement(hCurr);
						inhabs.removeElementAt(j);
						j--;
						progress=true;
					}
				}
				
			}
			persnohh += inhabs.size();
			
			for(int hhb=0; hhb<households.size(); hhb++) {
				if( !((Household) households.elementAt(hhb)).isComplete() ) {
					incomplete++;
					hhincomp++;
					hhincomplete[ (((Household) households.elementAt(hhb))).getHouseholdTypeID()-1 ]++;
				}
				else {
					hhcomp++;
					hhcomplete[ (((Household) households.elementAt(hhb))).getHouseholdTypeID()-1 ]++;
				}
				pershh += ((Household) households.elementAt(hhb)).getNumPerson();
			}
			//System.out.println("   "+inhabitants[i].size()+"  "+allHouseholdsOfBorough[i].size()+"   "+incomplete);
			
			if(!progress) noProgress++;
			System.out.println();
			System.out.println(noProgress+"   => "+hhcomp+"  "+hhincomp+"  "+pershh+"  "+persnohh);
			int[] c = new int[cohortSurvive.length];
			for (int j=0; j<inhabs.size(); j++) {
				c[ ((Inhabitant) inhabs.elementAt(j)).getCohort() ]++;
			}
		}
		
		for(Household h : households) {
			h.setInitialLastMoving();
		}
		
		return households;		
	}
	
	public void resetMigration() {
		for (int i=0; i<landParcelArray.length; i++){
			landParcelArray[i].resetMigration();
		}
	}
}
