package ufz.aloe.urban.model;

import java.util.Vector;

public class Household {
	
	private static HouseholdType[] householdTypeArray;
	private static float[][] budgetMatrix;
	private int id;
	private LandParcel landParcel;
	private Flat flat;
	private StructureType structureType;
	private boolean movingPossible;
	private int lastMovingHouse;
	private HouseholdType householdType;
	private Vector<Inhabitant> inhabitants;
	private int budget=3;
	
	public static void init(HouseholdType[] householdTypeArray, float[][] budget) {
		Household.householdTypeArray=householdTypeArray;
		Household.budgetMatrix=budget;
	}
	
	Household(
				int id, 
				LandParcel landParcel, 
				StructureType structureType, 
				HouseholdType householdType){
		this(id, householdType);
		this.landParcel = landParcel;
		this.structureType = structureType;
	}
	Household(
				int id, 
				HouseholdType householdType){
		this.id = id;
		this.householdType = householdType;
		inhabitants=new Vector<Inhabitant>();
		this.lastMovingHouse=(int)(Math.random()*householdType.getMinYearMove());
		updateLastMoving();
		calcBudget();
	}
	
	
	public void setInitialLastMoving() {
		this.lastMovingHouse=(int)(Math.random()*householdType.getMinYearMove());
	}
	
	public void calcBudget() {
		if(householdType!=null) {
			double r=Math.random();
			for(int i=0;i<budgetMatrix[0].length;i++) {
				if( budgetMatrix[i][getHouseholdTypeID()-1] >= r ) {
					budget=i;
					break;
				};
			}
		}
		else budget=0;
		//System.out.println("hht: "+getHouseholdTypeID()+",  budget: "+maxCost);
	}
	public void setFlat(Flat f) {
		this.flat=f;
		this.landParcel=f.getLandParcel();
		this.structureType = f.getStructureType();
	}
	public boolean moving (Flat moveTo){
			/* first checks if moving is possible
			 * then: searches the highest attractiveness land parcel with places to occupy
			 * 
			 * if the attractiveness is higher then the attractiveness of the current land parcel
			 * the household moves to this parcel
			 * and sends a message to the abandoned land parcel which reduces the number of 
			 * households and to the destination land parcel where the number of households is 
			 * augmented, lastMovingHouse is set to zero, moving possible is set to false
			 * 
			 * at the end the household number is deleted in the random search list
			 */

			Flat oldF=flat;
			if( moveTo.addHousehold(this) ) {
				if(oldF != null) oldF.removeHousehold(this);
				structureType=moveTo.getStructureType();
				landParcel = moveTo.getLandParcel();
				lastMovingHouse = 0;
				movingPossible = false;
				setFlat(moveTo);
				return true;	
			}
			else {
				return false;
			}
			//System.out.println("   to p "+landParcel.getID()+" s "+structureType.getID());	
			
	}
	public void removeFromModel() {
		if(flat != null) flat.removeHousehold(this);
		flat = null;
		structureType=null;
		landParcel = null;
		lastMovingHouse = 0;
	}

	public void updateLastMoving (){
		lastMovingHouse ++;
		if (lastMovingHouse >= householdType.getMinYearMove() ){
			movingPossible = true;
		}
		else movingPossible = false;
	}
	public int getLandParcelID() {
		return landParcel.getID();
	}
	public LandParcel getLandParcel() {
		return landParcel;
	}

	public HouseholdType getHouseholdType() {
		return householdType;
	}
	public Flat getFlat() {
		return flat;
	}
	public int getHouseholdTypeID() {
		if(householdType==null) {
			System.out.println("HHT null, EW: "+inhabitants.size());
			return 0;
		}
		return householdType.getID();
	}
	public int getBudget() {
		return budget;
	}
	public boolean movingPossible() {
		return movingPossible;
	}
	public int getNumPerson() {
		return inhabitants.size();
	}
	public Vector<Inhabitant> getInhabitants() {
		return inhabitants;
	}
	public void addPersonUnchecked(Inhabitant inh) {
		if(inhabitants==null) inhabitants=new Vector<Inhabitant>();
		inhabitants.addElement(inh);
	}
	public boolean addPerson(Inhabitant inh) {
		if(inhabitants==null) inhabitants=new Vector<Inhabitant>();
		if(hasMaxSize()) return false;
		if(inhabitants.size() > 0 
			&& isComplete() 
			&& Math.random() > HouseholdType.getHouseholdTypeConstraints()[20+inhabitants.size()][getHouseholdTypeID()-1]) 
					return false;
		switch (inhabitants.size()) {
			case 0:
					if(inh.getAge() >= HouseholdType.getHouseholdTypeConstraints()[0][getHouseholdTypeID()-1]
						&& inh.getAge() <= HouseholdType.getHouseholdTypeConstraints()[1][getHouseholdTypeID()-1] ) {
							inhabitants.addElement(inh);
							return true;
						}
					return false;
			case 1:
					Inhabitant i1 = inhabitants.elementAt(0);
					if(inh.getAge() >= HouseholdType.getHouseholdTypeConstraints()[2][getHouseholdTypeID()-1]
						&& inh.getAge() <= HouseholdType.getHouseholdTypeConstraints()[3][getHouseholdTypeID()-1]
						&& inh.getAge() >= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[4][getHouseholdTypeID()-1]
						&& inh.getAge() <= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[5][getHouseholdTypeID()-1]) {
							inhabitants.addElement(inh);
							return true;
						}
					return false;
			case 2:
					i1 = inhabitants.elementAt(0);
					Inhabitant i2 = inhabitants.elementAt(1);
					if(inh.getAge() >= HouseholdType.getHouseholdTypeConstraints()[6][getHouseholdTypeID()-1]
						&& inh.getAge() <= HouseholdType.getHouseholdTypeConstraints()[7][getHouseholdTypeID()-1]
						&& inh.getAge() >= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[8][getHouseholdTypeID()-1]
						&& inh.getAge() <= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[9][getHouseholdTypeID()-1]
						&& inh.getAge() >= i2.getAge()+HouseholdType.getHouseholdTypeConstraints()[10][getHouseholdTypeID()-1]
						&& inh.getAge() <= i2.getAge()+HouseholdType.getHouseholdTypeConstraints()[11][getHouseholdTypeID()-1]) {
							inhabitants.addElement(inh);
							return true;
						}
					return false;
			case 3:
					i1 = inhabitants.elementAt(0);
					i2 = inhabitants.elementAt(1);
					Inhabitant i3 = inhabitants.elementAt(2);
					if(inh.getAge() >= HouseholdType.getHouseholdTypeConstraints()[12][getHouseholdTypeID()-1]
						&& inh.getAge() <= HouseholdType.getHouseholdTypeConstraints()[13][getHouseholdTypeID()-1]
						&& inh.getAge() >= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[14][getHouseholdTypeID()-1]
						&& inh.getAge() <= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[15][getHouseholdTypeID()-1]
						&& inh.getAge() >= i2.getAge()+HouseholdType.getHouseholdTypeConstraints()[16][getHouseholdTypeID()-1]
						&& inh.getAge() <= i2.getAge()+HouseholdType.getHouseholdTypeConstraints()[17][getHouseholdTypeID()-1]
						&& inh.getAge() >= i3.getAge()+HouseholdType.getHouseholdTypeConstraints()[18][getHouseholdTypeID()-1]
						&& inh.getAge() <= i3.getAge()+HouseholdType.getHouseholdTypeConstraints()[19][getHouseholdTypeID()-1]) {
							inhabitants.addElement(inh);
							return true;
						}
					return false;
			default:
					i1 = inhabitants.elementAt(0);
					i2 = inhabitants.elementAt(1);
					i3 = inhabitants.elementAt(2);
					if(inh.getAge() >= HouseholdType.getHouseholdTypeConstraints()[12][getHouseholdTypeID()-1]
						&& inh.getAge() <= HouseholdType.getHouseholdTypeConstraints()[13][getHouseholdTypeID()-1]
						&& inh.getAge() >= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[14][getHouseholdTypeID()-1]
						&& inh.getAge() <= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[15][getHouseholdTypeID()-1]
						&& inh.getAge() >= i2.getAge()+HouseholdType.getHouseholdTypeConstraints()[16][getHouseholdTypeID()-1]
						&& inh.getAge() <= i2.getAge()+HouseholdType.getHouseholdTypeConstraints()[17][getHouseholdTypeID()-1]
						&& inh.getAge() >= i3.getAge()+HouseholdType.getHouseholdTypeConstraints()[18][getHouseholdTypeID()-1]
						&& inh.getAge() <= i3.getAge()+HouseholdType.getHouseholdTypeConstraints()[19][getHouseholdTypeID()-1]) {
							inhabitants.addElement(inh);
							return true;
						}
					return false;
		}
	}
	public boolean isComplete() {
		return inhabitants.size() >= HouseholdType.getPersonPerHousehold()[getHouseholdTypeID()-1];
	}
	public void deriveHouseholdType() {
		for(int i=0; i<householdTypeArray.length; i++) {
			switch (inhabitants.size()) {
				case 0:
						householdType=null;
						break;
				case 1:
						Inhabitant i1 = inhabitants.elementAt(0);
						if( inhabitants.size() >= HouseholdType.getPersonPerHousehold()[i]
							&& inhabitants.size() <= HouseholdType.getMaxPersonPerHousehold()[i]
							&& i1.getAge() >= HouseholdType.getHouseholdTypeConstraints()[0][i]
							&& i1.getAge() <= HouseholdType.getHouseholdTypeConstraints()[1][i] ) {
							
							householdType=householdTypeArray[i];
						}
						break;
				case 2:
						i1 = inhabitants.elementAt(0);
						Inhabitant i2 = inhabitants.elementAt(1);
						if(inhabitants.size() >= HouseholdType.getPersonPerHousehold()[i]
							&& inhabitants.size() <= HouseholdType.getMaxPersonPerHousehold()[i]
							&& i1.getAge() >= HouseholdType.getHouseholdTypeConstraints()[0][i]
							&& i1.getAge() <= HouseholdType.getHouseholdTypeConstraints()[1][i] 
							&& i2.getAge() >= HouseholdType.getHouseholdTypeConstraints()[2][i]
							&& i2.getAge() <= HouseholdType.getHouseholdTypeConstraints()[3][i]
							&& i2.getAge() >= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[4][i]
							&& i2.getAge() <= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[5][i]) {
							
							householdType=householdTypeArray[i];
						}
						break;	
				case 3:
						i1 = inhabitants.elementAt(0);
						i2 = inhabitants.elementAt(1);
						Inhabitant i3 = inhabitants.elementAt(2);
						if(inhabitants.size() >= HouseholdType.getPersonPerHousehold()[i]
							&& inhabitants.size() <= HouseholdType.getMaxPersonPerHousehold()[i]
							&& i1.getAge() >= HouseholdType.getHouseholdTypeConstraints()[0][i]
							&& i1.getAge() <= HouseholdType.getHouseholdTypeConstraints()[1][i] 
							&& i2.getAge() >= HouseholdType.getHouseholdTypeConstraints()[2][i]
							&& i2.getAge() <= HouseholdType.getHouseholdTypeConstraints()[3][i]
							&& i2.getAge() >= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[4][i]
							&& i2.getAge() <= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[5][i]
							&& i3.getAge() >= HouseholdType.getHouseholdTypeConstraints()[6][i]
							&& i3.getAge() <= HouseholdType.getHouseholdTypeConstraints()[7][i]
							&& i3.getAge() >= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[8][i]
							&& i3.getAge() <= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[9][i]
							&& i3.getAge() >= i2.getAge()+HouseholdType.getHouseholdTypeConstraints()[10][i]
							&& i3.getAge() <= i2.getAge()+HouseholdType.getHouseholdTypeConstraints()[11][i]) {
							
							householdType=householdTypeArray[i];
						}
						break;	
				default:
						i1 = inhabitants.elementAt(0);
						i2 = inhabitants.elementAt(1);
						i3 = inhabitants.elementAt(2);
						Inhabitant i4 = inhabitants.elementAt(3);
						if(inhabitants.size() >= HouseholdType.getPersonPerHousehold()[i]
							&& inhabitants.size() <= HouseholdType.getMaxPersonPerHousehold()[i]
							&& i1.getAge() >= HouseholdType.getHouseholdTypeConstraints()[0][i]
							&& i1.getAge() <= HouseholdType.getHouseholdTypeConstraints()[1][i] 
							&& i2.getAge() >= HouseholdType.getHouseholdTypeConstraints()[2][i]
							&& i2.getAge() <= HouseholdType.getHouseholdTypeConstraints()[3][i]
							&& i2.getAge() >= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[4][i]
							&& i2.getAge() <= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[5][i]
							&& i3.getAge() >= HouseholdType.getHouseholdTypeConstraints()[6][i]
							&& i3.getAge() <= HouseholdType.getHouseholdTypeConstraints()[7][i]
							&& i3.getAge() >= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[8][i]
							&& i3.getAge() <= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[9][i]
							&& i3.getAge() >= i2.getAge()+HouseholdType.getHouseholdTypeConstraints()[10][i]
							&& i3.getAge() <= i2.getAge()+HouseholdType.getHouseholdTypeConstraints()[11][i]
							&& i4.getAge() >= HouseholdType.getHouseholdTypeConstraints()[12][i]
							&& i4.getAge() <= HouseholdType.getHouseholdTypeConstraints()[13][i]
							&& i4.getAge() >= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[14][i]
							&& i4.getAge() <= i1.getAge()+HouseholdType.getHouseholdTypeConstraints()[15][i]
							&& i4.getAge() >= i2.getAge()+HouseholdType.getHouseholdTypeConstraints()[16][i]
							&& i4.getAge() <= i2.getAge()+HouseholdType.getHouseholdTypeConstraints()[17][i]
							&& i4.getAge() >= i3.getAge()+HouseholdType.getHouseholdTypeConstraints()[18][i]
							&& i4.getAge() <= i3.getAge()+HouseholdType.getHouseholdTypeConstraints()[19][i]) {
							
							householdType=householdTypeArray[i];
						}
						break;	
			}
		}
		calcBudget();
		//System.out.println(getHouseholdType());
	}
	public boolean hasMaxSize() {
		return inhabitants.size() >= HouseholdType.getMaxPersonPerHousehold()[getHouseholdTypeID()-1];
	}
	public Vector<Household> mature() {
		int nc=0;
		Vector<Household> ret= new Vector<Household>();
		Household hCurr=null;
		boolean hasChanged=false;
		int nrc=0;
		for(int i=0;i<inhabitants.size();i++) {
			int m=inhabitants.elementAt(i).mature();
			if(m==-1) {
				inhabitants.removeElementAt(i);
				i--;
				hasChanged=true;
			}
			else if(m>0)  {
				nc+=m;
				nrc=i;
			}
		}
		if(getHouseholdTypeID()==7 && nc>0) {
			hCurr=new Household(1, householdTypeArray[0]);
			hCurr.addPersonUnchecked( inhabitants.elementAt(nrc) );
			inhabitants.removeElementAt(nrc);
			while(nc>0) {
				hCurr.addPersonUnchecked( new Inhabitant(0,0) );
				nc--;
			}
			hCurr.deriveHouseholdType();
			hasChanged=true;
			ret.add(hCurr);	
		} else {
			while(nc>0) {
				addPersonUnchecked( new Inhabitant(0,0) );
				nc--;
				hasChanged=true;
			}
		}
		if(hasChanged) {
			deriveHouseholdType();
		}
		hasChanged=false;
		int n=0;
		//System.out.println(inhabitants.size()+"   "+getHouseholdType());
		//if(getHouseholdType()!=null) System.out.println(getHouseholdTypeID());
		for(Inhabitant i : inhabitants) {
			if(n>5) n=5;
			if(HouseholdType.getHouseholdTypeConstraints()[25+n][getHouseholdTypeID()-1]>1) {
				if(i.getAge()>=HouseholdType.getHouseholdTypeConstraints()[25+n][getHouseholdTypeID()-1]) {
					
					hCurr = new Household(1, householdTypeArray[0]);
					hCurr.addPersonUnchecked(i);
					hCurr.deriveHouseholdType();
					inhabitants.remove(i);
					n--;
					hasChanged=true;
					ret.add(hCurr);	
					break;
				}
			}
			else {
				if(Math.random() < HouseholdType.getHouseholdTypeConstraints()[25+n][getHouseholdTypeID()-1]) {
					hCurr = new Household(1, householdTypeArray[0]);
					hCurr.addPersonUnchecked(i);
					hCurr.deriveHouseholdType();
					inhabitants.remove(i);
					n--;
					hasChanged=true;
					ret.add(hCurr);	
					break;
				}	
			}
			n++;
			deriveHouseholdType();
			if(hasChanged) break;
		}
		if(hasChanged) deriveHouseholdType();
		return ret;
	}
	public Household copy() {
		Household ret = new Household(id, landParcel, structureType, householdType);
		
		ret.setMovingPossible(movingPossible);
		ret.setLastMovingHouse(lastMovingHouse);
		
		ret.budget=budget;
		
		Vector<Inhabitant> inh=new Vector<Inhabitant>();
		for(Inhabitant i : inhabitants) {
			inh.add(i.copy());
		}
		
		ret.setInhabitants(inh);
		
		return ret;
	}
	
	public void setMovingPossible(boolean movingPossible) {
		this.movingPossible=movingPossible;
	}
	
	public void setLastMovingHouse(int lastMovingHouse) {
		this.lastMovingHouse=lastMovingHouse;
	}
	public void setInhabitants(Vector<Inhabitant> inhabitants) {
		this.inhabitants=inhabitants;
	}
	public boolean equals(Object o) {
		return o==this;
	}
}
