package ufz.aloe.urban.model;
import java.util.Vector;
import java.lang.Math;
import sa.data.Raster;

public class LandParcel implements Comparable{

	private static StructureType[] structureTypeArray;
	private static HouseholdType [] householdTypeArray;
	
	private int ID;
	private double area;  // area (unit: square meter?)
	private int initialStructureType; // at the beginning the whole land parcel has the same structure type
	private int [] numberOfBuildings; // number of buildings is a vector of structure type
	private int maxNumberPlaces2Occupy; // maximum number of buildings or other places like schools, shops or parks..
	//private int [] numberOfFlats; // number of flats is a vector of structure type
	private int borough;			// input
//	private int [][] householdTypesInStructureTypes; // number of households for each structure type and each 
											   // household type, first index structure Type, second index for household type
	private int []vacancy_structureType; // vacancy flats as a vector of structure type
	//private Flat[] flatInStructureType; // Vector of StructureType
	private double []inhabitantsPerArea; // number of people per area unit (square meter) a vector of household type
	private float []attractiveness; // attractiveness is a vector of household type
	private int []migration;
	private float meanAttractiveness;
	private Vector householdVector;
	private static int sortField; 
	private Vector<Building> buildings;

	/**
	 * The land parcel constructor gets the following attributes from 
	 * input arrays of the UrbanModelParameters class:
	 *  landParcelID, area, initialStructureType, numberOfBuildings, borough
	 * 
	 * The number of flats results from the number of building multiplied by the number
	 * of flats per building typical for the structure type of the land parcel.
	 * 
	 * The initial distribution of household types and vacancy in the land parcel 
	 * is assigned as follows:
	 * 
	 * For each flat a household type (or vacancy) is randomly attributed according to the 
	 * data in UrbanModelParameter for the borough to which the land parcel belongs. 
	 * The data in UrbanModelParameter describe the percentages of household types and 
	 * vacancy for each borough.
	 * 
	 * 
	 * @param id index which refers to the ID of the landParcel constructed 
	 * (the input parameters are read from this line in the file landparcel.csv
	 * if lines are counted starting with zero)
	 */
	
	public static void init(StructureType []structureTypeArray,
			HouseholdType [] householdTypeArray){
		LandParcel.structureTypeArray=structureTypeArray;
		LandParcel.householdTypeArray=householdTypeArray;
		
	}
	
	public LandParcel (	int landParcelID, float area, int numberOfBuildings, 
			int initialStructureType, int borough,  Borough []boroughArray, 
			int[][] buildingPos, float[] proportionToBuy, Raster[] costs, 
			Raster greenery, int maxRangeGreenery, int greeneryClasses){

		this.ID= landParcelID;
		this.area= area;
		this.initialStructureType = initialStructureType;
		this.numberOfBuildings= new int [structureTypeArray.length];
		this.migration = new int [householdTypeArray.length];
		this.buildings = new Vector<Building>();
		for (int j=0; j<structureTypeArray.length;j++){
			this.numberOfBuildings[j]=0;
		}
		if(this.initialStructureType!=0) 
			this.numberOfBuildings[this.initialStructureType-1]=
			            numberOfBuildings;
		float buildingID = 0;
		int totNumberOfFlats = 0;
		for(int i = 0; i < buildingPos[0].length; i++) {
			//System.out.println(buildingPos[2][i]);
			if(buildingPos[2][i]==ID) {
				buildingID += 0.0001;
				Flat fl = new Flat(this, structureTypeArray[initialStructureType-1]);
				//System.out.println(i+"   "+initialStructureType);
				Building b = new Building(
									buildingPos[0][i],
									buildingPos[1][i],
									costs[initialStructureType-1],
									this, 
									structureTypeArray[initialStructureType-1], 
									(float)this.ID+buildingID,
									fl,
									structureTypeArray[initialStructureType-1].getFlatsPerBuilding(),
									(Math.random() < proportionToBuy[initialStructureType-1]),
									greenery,
									maxRangeGreenery,
									greeneryClasses
									); 
				
				this.buildings.add(b);
				
				totNumberOfFlats+=structureTypeArray[initialStructureType-1].getFlatsPerBuilding();	
			}
			
		}
		System.out.println( "parcel "+ID+":  "+buildings.size()+" buildings" );
		
		maxNumberPlaces2Occupy = numberOfBuildings;

		this.borough = borough;
		
		this.householdVector = new Vector();
		/*int totNumberOfFlats = 0;
	
		for (int j = 0; j < structureTypeArray.length; j++) {
			totNumberOfFlats += numberOfFlats[j];
		}*/	
		int totVacancy = 0;
		int[] totHouseholdTypes = new int[householdTypeArray.length];
			// Vector of household types
		for (int j = 0; j < householdVector.size(); j++) {
			Household hhcurr=(Household) householdVector.elementAt(j);
			totHouseholdTypes[hhcurr.getHouseholdTypeID()-1] += 1;
		}
	}	
	
	public void setHouseholds(Vector households) {
		this.householdVector = households;
		int totNumberOfFlats = 0;
		for(Building b : buildings) {
			totNumberOfFlats+=b.getNumberOfFlats();
		}
		/*for (int j = 0; j < structureTypeArray.length; j++) {
			totNumberOfFlats += numberOfFlats[j];
		}*/	
		int totVacancy = 0;
		int[] totHouseholdTypes = new int[householdTypeArray.length];
			// Vector of household types
		for (int j = 0; j < householdVector.size(); j++) {
			Household hhcurr=(Household) householdVector.elementAt(j);
			totHouseholdTypes[hhcurr.getHouseholdTypeID()-1] += 1;
		}

	}
		

			
	/**
	 * @return  void
	 * inhabitantsPerArea is simply calculated by the number of households of a household type 
	 * multiplied by the number of persons per household typical for that household type.
	 */
	
	public void calculateInhabitantsPerArea (HouseholdType[]  householdTypeArray, StructureType[]  structureTypeArray){
		//int households []= new int [householdTypeArray.length]; // number of households of the land parcel, vector of household type
		double a [] = new double [householdTypeArray.length];
		
		for(Building b : buildings) {
			Vector<Household> hh = b.getHouseholds();
			for(Household h : hh) {
				a[h.getHouseholdTypeID()-1]+=h.getNumPerson();
			}
		}
		for (int j =0; j < householdTypeArray.length; j++){ 
			a[j]/=area;
		}
		this.inhabitantsPerArea=a;
		
	}


	/**
	 * @return void
	 * 
	 * Attractiveness is determined only from the population densities of parcels within maximumPropertyDistances[City.SOCIAL_MILIEU] at the moment. 
	 * Household types differ in their preferences.  
	 * mean attractiveness is calculated asweighted average over inhabitants per household type over the attractivenesses for 
	 * the household types for this parcel. 
	 */
	public void calculateAttractiveness(
							HouseholdType[]  householdTypeArray, 
							LandParcel[] landParcelArray, 
							float[][] landParcelDistances,  
							Vector<Infrastructure> infrastructure,
							float[] maximumPropertyDistances,
							float[][] weightsOfParameters){
		attractiveness = new float[householdTypeArray.length];
		float inhabitantsPerAreaSum = 0;
		float a = 0;
		
		
		for (Building b: buildings){
			b.calculateAttractiveness( householdTypeArray, 
										landParcelArray, 
										landParcelDistances, 
										infrastructure, 
										maximumPropertyDistances,
										weightsOfParameters);
		}
	}

	  public static void setSortField(int i){ 
		    sortField = i; 
		  } 

	public int compareTo( Object other ){ 
		    /* hier werden jetzt zwei Objekte miteinander verglichen 
		       einmal this und other. die methode wird automatisch 
		       von Arrays aufgerufen ... 
		    */ 
		    
		    
		    int i=9;
			if (this.attractiveness[sortField]<((LandParcel)other).attractiveness[sortField])
				i=-1;
			 /* gendert: 
		     *	< => ==
		     */
		    if (this.attractiveness[sortField]==((LandParcel)other).attractiveness[sortField])
		    	i= 0;
			 /* gendert: 
		     *	< => >
		     */
		    if (this.attractiveness[sortField]>((LandParcel)other).attractiveness[sortField])
		    	i=1;
		    return i;
		   } 
	
		  

	/**
	 * @return Returns the area.
	 */
	public double getArea() {
		return area;
	}



	/**
	 * @return Returns the attractiveness.
	 */
	public float[] getAttractiveness() {
		return attractiveness;
	}



	/**
	 * @return Returns the borough id.
	 */
	public int getBorough() {
		return borough;
	}
	


	/**
	 * @return Returns the householdTypesInStructureTypes.
	 */
	/*public int[][] getHouseholdTypesInStructureTypes() {
		return householdTypesInStructureTypes;
	}*7



	/**
	 * @return Returns the householdVector.
	 */
	public Vector getHouseholdVector() {
		return householdVector;
	}



	/**
	 * @return Returns the iD.
	 */
	public int getID() {
		return ID;
	}



	/**
	 * @return Returns the inhabitantsPerArea.
	 */
	public double[] getInhabitantsPerArea() {
		return inhabitantsPerArea;
	}



	/**
	 * @return Returns the initialStructureType.
	 */
	public int getInitialStructureType() {
		return initialStructureType;
	}



	/**
	 * @return Returns the maxNumberPlaces2Occupy.
	 */
	public int getMaxNumberPlaces2Occupy() {
		return maxNumberPlaces2Occupy;
	}



	/**
	 * @return Returns the numberOfBuildings.
	 */
	public int[] getNumberOfBuildings() {
		return numberOfBuildings;
	}



	/**
	 * @return Returns the numberOfFlats.
	 */
	public int[] getNumberOfFlats() {
		int[] ret = new int[buildings.size()];
		int c=0;
		for(Building b : buildings) {
			ret[c]+=b.getNumberOfFlats();
			c++;
		}
		return ret;
	}




	public Flat[] getFlats() {
		Flat[] ret = new Flat[buildings.size()];
		int c=0;
		for(Building b : buildings) {
			ret[c]=b.getFlat();
			c++;
		}
		return ret;
	}
	public Vector<Building> getBuildings() {
		return buildings;
	}
	/**
	 * @return Returns the meanAttractiveness.
	 */
	public float getMeanAttractiveness() {
		return meanAttractiveness;
	}
	
	public void migrateIn(Household h) {
		if(h.getHouseholdTypeID()>0) migration[h.getHouseholdTypeID()-1]++;
	}
	public void migrateOut(Household h) {
		if(h.getHouseholdTypeID()>0) migration[h.getHouseholdTypeID()-1]--;
	}
	public int[] getMigration() {
		return migration;
	}
	public void resetMigration() {
		this.migration = new int [householdTypeArray.length];
	}

}
