package ufz.aloe.urban.preprocessing;


import java.util.*;
import java.io.*;

import ufz.aloe.urban.helper.*;


/**
 * controls the flow of actions
 * @param parametersRead //are all parameterfiles ok? can the model be run?
 */
public class DemographyOptimizer extends VectorConverter{
	
	private DemographyParameters[] para;
	
	
	private float[][] cohortSurviveBorder;
	private float[][] cohortZuBorder;
	private float[][] cohortWegBorder;
	private float[][] cohortBirthBorder;
	private float[] moveZuBorder;
	private float[] moveWegBorder;
	
	private float pCross;
	private float pMut, pMut2;
	private float range, range2;
	private int pop;
	private int steps;
	private float[] start;
	private float[] observed;
	private String name;
	private boolean block;


	public static void main(String[] args) throws IOException {
	
		
	
		Vector cohortSurvive=new Vector();
		Vector cohortZu=new Vector();
		Vector cohortWeg=new Vector();
		Vector cohortBirth=new Vector();
		Vector move=new Vector();
		Vector inhab=new Vector();
		
		cohortSurvive = readParameterFiles("opt/survive.csv");
		cohortBirth = readParameterFiles("opt/birth.csv");
		cohortZu = readParameterFiles("opt/cohortZu.csv");
		cohortWeg = readParameterFiles("opt/cohortWeg.csv");
		move = readParameterFiles("opt/move.csv");
		inhab = readParameterFiles("opt/inhabitants.csv");
	
		
		new DemographyOptimizer(
						cohortSurvive,
						cohortZu,
						cohortWeg,
						cohortBirth,
						move,
						inhab);
		
						
		
	}
	
	public DemographyOptimizer(
						Vector cohortSurvive,
						Vector cohortZu,
						Vector cohortWeg,
						Vector cohortBirth,
						Vector move,
						Vector inhab) 
	{
		pCross=0.3f;
		pMut=0.01f;
		pMut2=0.002f;
		pop=25;
		steps=1;
		range=0.05f;
		range2=0.2f;
		name="free_04_05";
		block = true;
		//name="noblock";
		
		cohortSurviveBorder=getFloatMatrix(1,cohortSurvive);
		cohortBirthBorder=getFloatMatrix(1,cohortBirth);
		cohortZuBorder=getFloatMatrix(1,cohortZu);
		cohortWegBorder=getFloatMatrix(1,cohortWeg);
		moveZuBorder=getFloatArray(1,move);
		moveWegBorder=getFloatArray(2,move);
		start=getFloatArray(1,inhab);
		observed=getFloatArray(2,inhab);
		
		DemographyParameters.init(
									cohortSurviveBorder,
									cohortZuBorder,
									cohortWegBorder,
									cohortBirthBorder,
									moveZuBorder,
									moveWegBorder,
									start,
									observed,
									pCross,
									pMut,
									pMut2,
									steps,
									range,
									range2,
									name );
		
		System.out.println("initializing population");
		
		para=new DemographyParameters[pop];
		for(int i=0; i<para.length; i++) {
			System.out.println("initializing "+i);
			
			float[] cohortSurviveP=new float[cohortSurviveBorder[0].length];
			float[] cohortZuP=new float[cohortSurviveBorder[0].length];
			float[] cohortWegP=new float[cohortSurviveBorder[0].length];
			float[] cohortBirthP=new float[cohortSurviveBorder[0].length];
			float moveZuP=0;
			float moveWegP=0;
			
			float sumZu =0;
			float sumWeg =0;
			
			for(int j=0; j<cohortSurviveP.length; j++) {
				cohortSurviveP[j]=cohortSurviveBorder[0][j]+ (float) Math.random()*cohortSurviveBorder[1][j];
				cohortZuP[j]=cohortZuBorder[0][j]+ (float) Math.random()*cohortZuBorder[1][j];
				cohortWegP[j]=cohortWegBorder[0][j]+ (float) Math.random()*cohortWegBorder[1][j];
				cohortBirthP[j]=cohortBirthBorder[0][j]+ (float) Math.random()*cohortBirthBorder[1][j];
				sumZu += cohortZuP[j];
				sumWeg += cohortWegP[j];
				//System.out.println(cohortZuP[j]+"     "+cohortWegP[j]);
			}
			for(int j = 0; j < cohortWegP.length; j++) {
				cohortWegP[j] *= 100/sumWeg;
				cohortZuP[j] *= 100/sumZu;
			}
			moveZuP=moveZuBorder[0]+ (float) Math.random()*moveZuBorder[1];
			moveWegP=moveWegBorder[0]+ (float) Math.random()*moveWegBorder[1];
			para[i] = new DemographyParameters(
									cohortSurviveP,
									cohortZuP,
									cohortWegP,
									cohortBirthP,
									moveZuP,
									moveWegP
									);
			
		}
		
		System.out.println("starting optimization");
		
		run(1000000);
		
		
	}

	public void runMC(int runs) {
		DemographyParameters curr;
		DemographyParameters best=selectBest();
		for(int i=0; i<runs; i++) {
			
			float[] cohortSurviveP=new float[cohortSurviveBorder[0].length];
			float[] cohortZuP=new float[cohortSurviveBorder[0].length];
			float[] cohortWegP=new float[cohortSurviveBorder[0].length];
			float[] cohortBirthP=new float[cohortSurviveBorder[0].length];
			float moveZuP=0;
			float moveWegP=0;
			for(int j=0; j<cohortSurviveP.length; j++) {
				cohortSurviveP[j]=cohortSurviveBorder[0][j]+ (float) Math.random()*cohortSurviveBorder[1][j];
				cohortZuP[j]=cohortZuBorder[0][j]+ (float) Math.random()*cohortZuBorder[1][j];
				cohortWegP[j]=cohortWegBorder[0][j]+ (float) Math.random()*cohortWegBorder[1][j];
				cohortBirthP[j]=cohortBirthBorder[0][j]+ (float) Math.random()*cohortBirthBorder[1][j];
				
			}
			moveZuP=moveZuBorder[0]+ (float) Math.random()*moveZuBorder[1];
			moveWegP=moveWegBorder[0]+ (float) Math.random()*moveWegBorder[1];
			curr = new DemographyParameters(
									cohortSurviveP,
									cohortZuP,
									cohortWegP,
									cohortBirthP,
									moveZuP,
									moveWegP
									);
			if(curr.getScore() > best.getScore() ) {
				best=curr;
				best.print();
			}
			System.out.println(i+"    "+best.getScore());
			
		}
	}

	public void run(int runs) {
		writeTable(new int[0][0], new File("opt/"+name+"_progress.csv"));
		for(int i=0; i<runs; i++) {
			DemographyParameters[] parents= selectParents( i%3 != 0 );
			//DemographyParameters child 
			//	= new DemographyParameters(parents[0], parents[1], true, true);
			DemographyParameters child 
				= new DemographyParameters(parents[0], parents[1], block);
			insert(child, true);	
			DemographyParameters best = selectBest();
			DemographyParameters worst = selectWorst();
			best.print();
			float sum=0;
			for(int j = 0; j < para.length; j++) {
				sum += para[j].getScore();
			}
			sum /= para.length;
			System.out.println(i+" "+sum+"  "+best.getScore()+" "+worst.getScore()+"  "+child.getScore());	
			float[] f = {
				i,
				sum,
				best.getScore(), 
				worst.getScore(), 
				parents[0].getScore(), 
				parents[1].getScore(), 
				child.getScore()
			};
			
			appendToTable(f, new File("opt/"+name+"_progress.csv"));
		}
	}

	public DemographyParameters[] selectParents(boolean byRank) {
		if(byRank) {
			float sum=0;
			float[] sc=new float[pop];
			DemographyParameters[] ret = new DemographyParameters[2];
			Vector sort=new Vector();
			for(int i=0; i<pop; i++) {
				int rank=0;
				for(int j=0; j<sort.size(); j++) {
					if( ((DemographyParameters) sort.elementAt(j)).getScore()>((DemographyParameters) para[i]).getScore() ) {
						rank++;
					}
				}
				sort.insertElementAt(para[i], rank);
			}
			
			for(int i=0; i<sort.size(); i++) {
				sum += 1/(i+1);
			}
			float ran1 = (float) Math.random()*sum;
			float ran2 = (float) Math.random()*sum;
			sum=0;
			for(int i=0; i<sort.size(); i++) {
				sum += 1/(i+1);
				if(sum >= ran1) {
					ret[0]=(DemographyParameters) sort.elementAt(i);
					break;
				}
			}
			sum=0;
			for(int i=0; i<sort.size(); i++) {
				sum += 1/(i+1);
				if(sum >= ran2) {
					ret[1]=(DemographyParameters) sort.elementAt(i);
					break;
				}
			}
			
			return ret;
			
		}
		else {
			float sum=0;
			DemographyParameters[] ret = new DemographyParameters[2];
			for(int i=0; i<para.length; i++) {
				sum += para[i].getScore()*para[i].getScore();
			}
			float ran1 = (float) Math.random()*sum;
			float ran2 = (float) Math.random()*sum;
			sum=0;
			for(int i=0; i<para.length; i++) {
				sum += para[i].getScore()*para[i].getScore();
				if(sum >= ran1) {
					ret[0]=para[i];
					break;
				}
			}
			sum=0;
			for(int i=0; i<para.length; i++) {
				sum += para[i].getScore()*para[i].getScore();
				if(sum >= ran2) {
					ret[1]=para[i];
					break;
				}
			}
			return ret;
		}
	}

	public void insert(DemographyParameters child, boolean once) {
		if(once) {
			float min=para[0].getScore();
			int nr=0;
			for(int i=0; i<para.length; i++) {
				if(para[i].getScore() < min) {
					min=para[i].getScore();
					nr=i;
				}
			}
			if( para[nr].getScore() <= child.getScore() ) para[nr] = child;
		}
		else {
			for(int i=0; i<para.length; i++) {
				if( para[i].getScore() < child.getScore() ) para[i] = child;
			}
			
		}
			
	}

	public DemographyParameters selectBest() {
		float max=para[0].getScore();
		int nr=0;
		for(int i=0; i<para.length; i++) {
			if(para[i].getScore() > max) {
				max=para[i].getScore();
				nr=i;
			}
		}
		return para[nr];
	}
	public DemographyParameters selectWorst() {
		float min=para[0].getScore();
		int nr=0;
		for(int i=0; i<para.length; i++) {
			if(para[i].getScore() < min) {
				min=para[i].getScore();
				nr=i;
			}
		}
		return para[nr];
	}

	public static Vector readParameterFiles(String inputFile){
		Vector inputVec = new Vector();
		try {
			/**
			 * Reads the parameter files in the input folder
			 *  @param inputFile String that conatins the path to the parameter file
			 */
			
			FileReader in = new FileReader(inputFile);
			CsvReader csv = new CsvReader(in);
			inputVec = csv.readCsvFile();

		//	System.out.println(inputVec.size());
		//	System.out.println(inputVec);
		
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return inputVec;
	}
	public static void printMatrix(int[][] m) {
		for(int i = 0; i < m.length; i++) {
			for(int j = 0; j < m[0].length; j++) {
				System.out.print(m[i][j]+"  ");
			}
			System.out.println("");
		}
	}
	public static void printArray(int[] m) {
		for(int i = 0; i < m.length; i++) {
			System.out.print(m[i]+"  ");
		}	
		System.out.println("");
	}
	public static void writeTable(int[][] d, File f) {
			StringWriter buffer = new StringWriter();
			PrintWriter out = new PrintWriter( buffer );
			for(int i = 0; i < d.length; i++) {	
				for(int j = 0; j < d[0].length; j++) {	
					out.print(d[i][j]+";");
				}
				out.println("");
			}
			
	    	String toWrite = buffer.toString();
	
	    	
	    	try {
	    		FileWriter fw = new FileWriter( f );
	    		fw.write( toWrite );
	    		fw.close();
	    	} catch ( IOException e ) {
	      		System.out.println( "Konnte Datei nicht erstellen" );
	    	}
	  
	}
	public static void appendToTable(float[] d, File f) {
		FileOutputStream out;
		try {
			out = new FileOutputStream(f, true);
			String lineSep = System.getProperty( "line.separator" );
			for(int i = 0; i < d.length; i++) {	
				out.write(new String(d[i]+";").getBytes());	
			}
			out.write(lineSep.getBytes());
			out.close();
		} catch ( IOException e ) {
      		e.printStackTrace(System.out);
    	}	
	}

}
