package ufz.aloe.urban.preprocessing;


import java.util.*;
import java.io.*;

import ufz.aloe.urban.helper.*;
import ufz.aloe.urban.model.*;


/**
 * controls the flow of actions
 * @param parametersRead //are all parameterfiles ok? can the model be run?
 */
public class DemographyParameters extends VectorConverter{
	
	private float[] cohortSurvive;
	private float[] cohortZu;
	private float[] cohortWeg;
	private float[] cohortBirth;
	private float moveZu;
	private float moveWeg;
	private float[] predicted;
	
	private float error;
	private float score;
	private Random random = new Random();
	
	public static float[][] cohortSurviveBorder;
	public static float[][] cohortZuBorder;
	public static float[][] cohortWegBorder;
	public static float[][] cohortBirthBorder;
	public static float[] moveZuBorder;
	public static float[] moveWegBorder;
	
	public static int steps;
	public static float pCross;
	public static float pMut,pMut2;
	public static float range,range2;
	public static float[] start;
	public static float[] observed;
	public static  String name;

	public static void init(
						float[][] cohortSurviveBorder,
						float[][] cohortZuBorder,
						float[][] cohortWegBorder,
						float[][] cohortBirthBorder,
						float[] moveZuBorder,
						float[] moveWegBorder,
						float[] start,
						float[] observed,
						float pCross,
						float pMut,
						float pMut2,
						int steps,
						float range,
						float range2,
						String name) {
		
		DemographyParameters.cohortSurviveBorder=cohortSurviveBorder;
		DemographyParameters.cohortZuBorder=cohortZuBorder;
		DemographyParameters.cohortWegBorder=cohortWegBorder;
		DemographyParameters.cohortBirthBorder=cohortBirthBorder;
		DemographyParameters.moveZuBorder=moveZuBorder;
		DemographyParameters.moveWegBorder=moveWegBorder;
		DemographyParameters.start=start;
		DemographyParameters.observed=observed;
		DemographyParameters.pCross=pCross;
		DemographyParameters.pMut=pMut;
		DemographyParameters.pMut2=pMut2;
		DemographyParameters.steps=steps;
		DemographyParameters.range=range;
		DemographyParameters.range2=range2;
		DemographyParameters.name=name;
		
	}
	
	public DemographyParameters(
				float[] cohortSurvive,
				float[] cohortZu,
				float[] cohortWeg,
				float[] cohortBirth,
				float moveZu,
				float moveWeg) 
	{
		
		this.cohortSurvive=cohortSurvive;
		this.cohortZu=cohortZu;
		this.cohortWeg=cohortWeg;
		this.cohortBirth=cohortBirth;
		this.moveZu=moveZu;
		this.moveWeg=moveWeg;
		run();
	}
	
	public DemographyParameters(DemographyParameters para1, DemographyParameters para2, boolean cross, boolean crossBlock) 
	{
		
		this.cohortSurvive=para1.cohortSurvive;
		this.cohortZu=para1.cohortZu;
		this.cohortWeg=para1.cohortWeg;
		this.cohortBirth=para1.cohortBirth;
		this.moveZu=para1.moveZu;
		this.moveWeg=para1.moveWeg;
		
		if(Math.random() > pCross) {
			if(Math.random() < 0.5) {
				this.cohortSurvive=para2.cohortSurvive;
				this.cohortZu=para2.cohortZu;
				this.cohortWeg=para2.cohortWeg;
				this.cohortBirth=para2.cohortBirth;
				this.moveZu=para2.moveZu;
				this.moveWeg=para2.moveWeg;
			}
				for(int i = 0; i < cohortSurvive.length; i++) {
					if(Math.random() <= pMut) cohortSurvive[i]=cohortSurviveBorder[0][i]+cohortSurviveBorder[1][i]* (float) Math.random();
					
				}
				float sumZu=0;
				float sumWeg=0;
				for(int i = 0; i < cohortZu.length; i++) {
					if(Math.random() <= pMut) cohortZu[i]=cohortZuBorder[0][i]+(cohortZuBorder[1][i]* (float) Math.random());
					sumZu += cohortZu[i];
				}
				for(int i = 0; i < cohortZu.length; i++) {
					cohortZu[i] *= 100/sumZu;
				}
				for(int i = 0; i < cohortWeg.length; i++) {
					if(Math.random() <= pMut) cohortWeg[i]=cohortWegBorder[0][i]+(cohortWegBorder[1][i]* (float) Math.random());
					sumWeg += cohortWeg[i];
				}
				for(int i = 0; i < cohortWeg.length; i++) {
					cohortWeg[i] *= 100/sumWeg;
				}
				for(int i = 0; i < cohortBirth.length; i++) {
					if(Math.random() <= pMut) cohortBirth[i]=cohortBirthBorder[0][i]+cohortBirthBorder[1][i]* (float) Math.random();
				}
				if(Math.random() <= pMut) moveZu=moveZuBorder[0]+moveZuBorder[1]* (float) Math.random();		
				if(Math.random() <= pMut) moveWeg=moveWegBorder[0]+moveWegBorder[1]* (float) Math.random();
		}
		else {		
			if(cross) {	
				if(crossBlock)	{
					if(Math.random() > 0.5) cohortSurvive=para2.cohortSurvive;
					for(int i = 0; i < cohortSurvive.length; i++) {
						if(Math.random() <= pMut) cohortSurvive[i]=cohortSurviveBorder[0][i]+cohortSurviveBorder[1][i]* (float) Math.random();
					}
					float sumZu=0;
					float sumWeg=0;
					if(Math.random() > 0.5) cohortZu=para2.cohortZu;
					for(int i = 0; i < cohortZu.length; i++) {
						if(Math.random() <= pMut) cohortZu[i]=cohortZuBorder[0][i]+(cohortZuBorder[1][i]* (float) Math.random());
						sumZu += cohortZu[i];
					}
					for(int i = 0; i < cohortZu.length; i++) {
						cohortZu[i] *= 100/sumZu;
					}
					if(Math.random() > 0.5) cohortWeg=para2.cohortWeg;
					for(int i = 0; i < cohortWeg.length; i++) {
						if(Math.random() <= pMut) cohortWeg[i]=cohortWegBorder[0][i]+cohortWegBorder[1][i]* (float) Math.random();
						sumWeg += cohortWeg[i];
					}
					for(int i = 0; i < cohortWeg.length; i++) {
						cohortWeg[i] *= 100/sumWeg;
					}
					if(Math.random() > 0.5) cohortBirth=para2.cohortBirth;
					for(int i = 0; i < cohortBirth.length; i++) {
						if(Math.random() <= pMut) cohortBirth[i]=cohortBirthBorder[0][i]+cohortBirthBorder[1][i]* (float) Math.random();
					}
					if(Math.random() > 0.5) moveZu=para2.moveZu;
					if(Math.random() <= pMut) moveZu=moveZuBorder[0]+moveZuBorder[1]* (float) Math.random();		
					if(Math.random() > 0.5) moveWeg=para2.moveWeg;
					if(Math.random() <= pMut) moveWeg=moveWegBorder[0]+moveWegBorder[1]* (float) Math.random();
				}
				else {
					for(int i = 0; i < cohortSurvive.length; i++) {
						if(Math.random() > 0.5) cohortSurvive[i]=para2.cohortSurvive[i];
						if(Math.random() <= pMut) cohortSurvive[i]=cohortSurviveBorder[0][i]+cohortSurviveBorder[1][i]* (float) Math.random();
					}
					float sumZu=0;
					float sumWeg=0;
					for(int i = 0; i < cohortZu.length; i++) {
						if(Math.random() > 0.5) cohortZu[i]=para2.cohortZu[i];
						if(Math.random() <= pMut) cohortZu[i]=cohortZuBorder[0][i]+(cohortZuBorder[1][i]* (float) Math.random());
						sumZu += cohortZu[i];
					}
					for(int i = 0; i < cohortZu.length; i++) {
						cohortZu[i] *= 100/sumZu;
					}
					for(int i = 0; i < cohortWeg.length; i++) {
						if(Math.random() > 0.5) cohortWeg[i]=para2.cohortWeg[i];
						if(Math.random() <= pMut) cohortWeg[i]=cohortWegBorder[0][i]+cohortWegBorder[1][i]* (float) Math.random();
						sumWeg += cohortWeg[i];
					}
					for(int i = 0; i < cohortWeg.length; i++) {
						cohortWeg[i] *= 100/sumWeg;
					}
					for(int i = 0; i < cohortBirth.length; i++) {
						if(Math.random() > 0.5) cohortBirth[i]=para2.cohortBirth[i];
						if(Math.random() <= pMut) cohortBirth[i]=cohortBirthBorder[0][i]+cohortBirthBorder[1][i]* (float) Math.random();
					}
					if(Math.random() > 0.5) moveZu=para2.moveZu;
					if(Math.random() <= pMut) moveZu=moveZuBorder[0]+moveZuBorder[1]* (float) Math.random();		
					if(Math.random() > 0.5) moveWeg=para2.moveWeg;
					if(Math.random() <= pMut) moveWeg=moveWegBorder[0]+moveWegBorder[1]* (float) Math.random();
				}
				
			}
			else {
				for(int i = 0; i < cohortSurvive.length; i++) {
					cohortSurvive[i]= (para1.cohortSurvive[i]+para2.cohortSurvive[i])/2;
					if(Math.random() <= pMut) cohortSurvive[i]=cohortSurviveBorder[0][i]+cohortSurviveBorder[1][i]* (float) Math.random();
				}
				float sumZu=0;
				float sumWeg=0;
				for(int i = 0; i < cohortZu.length; i++) {
					cohortZu[i]=(para1.cohortZu[i]+para2.cohortZu[i])/2;
					if(Math.random() <= pMut) cohortZu[i]=cohortZuBorder[0][i]+cohortZuBorder[1][i]* (float) Math.random();
					sumZu += cohortZu[i];
				}
				for(int i = 0; i < cohortZu.length; i++) {
					cohortZu[i] *= 100/sumZu;
				}
				for(int i = 0; i < cohortWeg.length; i++) {
					cohortWeg[i]=(para1.cohortWeg[i]+para2.cohortWeg[i])/2;
					if(Math.random() <= pMut) cohortWeg[i]=cohortWegBorder[0][i]+cohortWegBorder[1][i]* (float) Math.random();
					sumWeg += cohortWeg[i];
				}
				for(int i = 0; i < cohortWeg.length; i++) {
					cohortWeg[i] *= 100/sumWeg;
				}
				for(int i = 0; i < cohortBirth.length; i++) {
					cohortBirth[i]=(para1.cohortBirth[i]+para2.cohortBirth[i])/2;
					if(Math.random() <= pMut) cohortBirth[i]=cohortBirthBorder[0][i]+cohortBirthBorder[1][i]* (float) Math.random();
				}
				moveZu=(para1.moveZu+para2.moveZu)/2;
				if(Math.random() <= pMut) moveZu=moveZuBorder[0]+moveZuBorder[1]* (float) Math.random();		
				moveWeg=(para1.moveWeg+para2.moveWeg)/2;
				if(Math.random() <= pMut) moveWeg=moveWegBorder[0]+moveWegBorder[1]* (float) Math.random();
			}
		}
		run();
		
	}
	
	public DemographyParameters(DemographyParameters para1, DemographyParameters para2, boolean block) 
	{
		
		this.cohortSurvive=para1.cohortSurvive;
		this.cohortZu=para1.cohortZu;
		this.cohortWeg=para1.cohortWeg;
		this.cohortBirth=para1.cohortBirth;
		this.moveZu=para1.moveZu;
		this.moveWeg=para1.moveWeg;
		
		if(Math.random() > pCross) {
			if(Math.random() < 0.5) {
				this.cohortSurvive=para2.cohortSurvive;
				this.cohortZu=para2.cohortZu;
				this.cohortWeg=para2.cohortWeg;
				this.cohortBirth=para2.cohortBirth;
				this.moveZu=para2.moveZu;
				this.moveWeg=para2.moveWeg;
			}
				
				
		}
		else {		
			if(block){
				if(Math.random() > pCross) cohortSurvive=para2.cohortSurvive;
				if(Math.random() > pCross) cohortZu=para2.cohortZu;
				if(Math.random() > pCross) cohortWeg=para2.cohortWeg;
				if(Math.random() > pCross) cohortBirth=para2.cohortBirth;
				
			}
			else {
				for(int i = 0; i < cohortSurvive.length; i++) { 
					if(Math.random() < pCross) cohortSurvive[i]=para2.cohortSurvive[i];
				}
				for(int i = 0; i < cohortZu.length; i++) {
					if(Math.random() < pCross) cohortWeg[i]=para2.cohortWeg[i];
				}
				for(int i = 0; i < cohortWeg.length; i++) {
					if(Math.random() < pCross) cohortWeg[i]=para2.cohortWeg[i];
				}
				for(int i = 0; i < cohortBirth.length; i++) {
					if(Math.random() < pCross) cohortBirth[i]=para2.cohortBirth[i];
				}
			}
			if(Math.random() < pCross) moveZu=para2.moveZu;
			if(Math.random() < pCross) moveWeg=para2.moveWeg;
			
			
		}
		for(int i = 0; i < cohortSurvive.length; i++) {
				if(Math.random() <= pMut) cohortSurvive[i] += cohortSurviveBorder[1][i]*range* random.nextGaussian();
				if(Math.random() <= pMut2) cohortSurvive[i] += cohortSurviveBorder[1][i]*range2* random.nextGaussian();
				if(cohortSurvive[i] < cohortSurviveBorder[0][i]) cohortSurvive[i] = cohortSurviveBorder[0][i];
				if(cohortSurvive[i] > cohortSurviveBorder[0][i]+cohortSurviveBorder[1][i]) cohortSurvive[i]=cohortSurviveBorder[0][i]+cohortSurviveBorder[1][i];
			}
		float sumZu=0;
		float sumWeg=0;
		for(int i = 0; i < cohortZu.length; i++) {
				
				if(Math.random() <= pMut) cohortZu[i] += cohortZuBorder[1][i]*range* random.nextGaussian();
				if(Math.random() <= pMut2) cohortZu[i] += cohortZuBorder[1][i]*range2* random.nextGaussian();
				if(cohortZu[i] < cohortZuBorder[0][i]) cohortZu[i] = cohortZuBorder[0][i];
				if(cohortZu[i] > cohortZuBorder[0][i]+cohortZuBorder[1][i]) cohortZu[i]=cohortZuBorder[0][i]+cohortZuBorder[1][i];
				sumZu += cohortZu[i];
			}
		for(int i = 0; i < cohortZu.length; i++) {
			cohortZu[i] *= 100/sumZu;
		}
		for(int i = 0; i < cohortWeg.length; i++) {
				if(Math.random() <= pMut) cohortWeg[i] += cohortWegBorder[1][i]*range* random.nextGaussian();
				if(Math.random() <= pMut2) cohortWeg[i] += cohortWegBorder[1][i]*range2* random.nextGaussian();
				if(cohortWeg[i] < cohortWegBorder[0][i]) cohortWeg[i] = cohortWegBorder[0][i];
				if(cohortWeg[i] > cohortWegBorder[0][i]+cohortWegBorder[1][i]) cohortWeg[i]=cohortWegBorder[0][i]+cohortWegBorder[1][i];
				sumWeg += cohortWeg[i];
			}
		for(int i = 0; i < cohortWeg.length; i++) {
			cohortWeg[i] *= 100/sumWeg;
		}
		for(int i = 0; i < cohortBirth.length; i++) {
				if(Math.random() <= pMut) cohortBirth[i] += cohortBirthBorder[1][i]*range* random.nextGaussian();
				if(Math.random() <= pMut2) cohortBirth[i] += cohortBirthBorder[1][i]*range2* random.nextGaussian();
				if(cohortBirth[i] < cohortBirthBorder[0][i]) cohortBirth[i] = cohortBirthBorder[0][i];
				if(cohortBirth[i] > cohortBirthBorder[0][i]+cohortBirthBorder[1][i]) cohortBirth[i]=cohortBirthBorder[0][i]+cohortBirthBorder[1][i];
			}
		if(Math.random() <= pMut) moveZu += moveZuBorder[1]*range* random.nextGaussian();
		if(Math.random() <= pMut2) moveZu += moveZuBorder[1]*range2* random.nextGaussian();
				if(moveZu < moveZuBorder[0]) moveZu = moveZuBorder[0];
				if(moveZu > moveZuBorder[0]+moveZuBorder[1]) moveZu=moveZuBorder[0]+moveZuBorder[1];
	
		if(Math.random() <= pMut) moveWeg += moveWegBorder[1]*range* random.nextGaussian();
		if(Math.random() <= pMut2) moveWeg += moveWegBorder[1]*range2* random.nextGaussian();
				if(moveWeg < moveWegBorder[0]) moveWeg = moveWegBorder[0];
				if(moveWeg > moveWegBorder[0]+moveWegBorder[1]) moveWeg=moveWegBorder[0]+moveWegBorder[1];
	
		run();
		
	}

	public void run() {
		Inhabitant.init(cohortSurvive, cohortBirth, 5);
		
		Vector inhabitants = new Vector();
		for(int coh = 0; coh < start.length; coh++) {
			for(int n = 0; n < start[coh]; n++) {
				inhabitants.addElement(new Inhabitant(coh, (int)(Math.random()*5) ) );
			}
		}
		for(int t = 0; t < steps; t++) {	
			Vector inhabitantsAdd = new Vector();
			predicted=new float[cohortSurvive.length];
			float[] inhabZu=new float[cohortSurvive.length];
			float[] inhabWeg=new float[cohortSurvive.length];
			for(int i = 0; i < inhabitants.size(); i++) {
				Inhabitant inhab = (Inhabitant) inhabitants.elementAt(i);
				
				int res = inhab.mature();
				if(res==-1) {
					inhabitants.removeElementAt(i);
					i--;
				} 
				else {
					if(res==1) {
						inhabitantsAdd.addElement( new Inhabitant( 0, 0 ) );
					}
				}
			}
			for(int i =0; i<inhabZu.length; i++) {
				inhabZu[i]=(cohortZu[i]*moveZu)/100;
				inhabWeg[i]=((cohortWeg[i]*moveWeg)/100)/start[i];		
			}
			for(int rem =0; rem<inhabitants.size(); rem++) {
				Inhabitant inhab = (Inhabitant) inhabitants.elementAt(rem);
				if( Math.random() <= inhabWeg[inhab.getCohort()] ) {
					inhabitants.removeElementAt(rem);
					rem--;
				}
			}
			for(int i =0; i<inhabZu.length; i++) {
				for(int add =0; add<inhabZu[i]; add++) 
						inhabitantsAdd.addElement(new Inhabitant(i, (int)(Math.random()*5) ) );
				
			}
			for(int a = 0; a < inhabitantsAdd.size(); a++) {
				inhabitants.addElement(inhabitantsAdd.elementAt(a));
			}
			
			
			
			for(int i = 0; i < inhabitants.size(); i++) {
				Inhabitant inhab = (Inhabitant) inhabitants.elementAt(i);
				predicted[inhab.getCohort()]++;
			}
		}
		error=0;
		/*for(int i = 0; i < predicted.length; i++) {
			error += Math.abs(predicted[i]-observed[i]);
		}*/
		for(int i = 0; i < predicted.length; i++) {
			error += Math.abs( Math.pow(predicted[i]-observed[i],2) );
		}
		/*for(int i = 0; i < predicted.length; i++) {
			if(error < Math.abs(predicted[i]-observed[i])) error = Math.abs(predicted[i]-observed[i]);
			
		}*/
		
		score=1/error;
		//System.out.println(score);	
	}

	public float getScore() {
		return score;
	}

	public void print() {
		float[] mv = { moveZu, moveWeg };
		
		writeTable(cohortSurvive, new File("opt/"+name+"_survive_out.csv"));
		writeTable(cohortBirth, new File("opt/"+name+"_birth_out.csv"));
		writeTable(cohortZu, new File("opt/"+name+"_cohortZu_out.csv"));
		writeTable(cohortWeg, new File("opt/"+name+"_cohortWeg_out.csv"));
		writeTable(predicted, new File("opt/"+name+"_predicted_out.csv"));
		writeTable(mv, new File("opt/"+name+"_move_out.csv"));
	}
	
	public static void writeTable(int[][] d, File f) {
		StringWriter buffer = new StringWriter();
		PrintWriter out = new PrintWriter( buffer );
		for(int i = 0; i < d.length; i++) {	
			for(int j = 0; j < d[0].length; j++) {	
				out.print(d[i][j]+";");
			}
			out.println("");
		}
		
    	String toWrite = buffer.toString();

    	
    	try {
    		FileWriter fw = new FileWriter( f );
    		fw.write( toWrite );
    		fw.close();
    	} catch ( IOException e ) {
      		System.out.println( "Konnte Datei nicht erstellen" );
    	}
	  
	}
	public static void writeTable(float[][] d, File f) {
		StringWriter buffer = new StringWriter();
		PrintWriter out = new PrintWriter( buffer );
		for(int i = 0; i < d.length; i++) {	
			for(int j = 0; j < d[0].length; j++) {	
				out.print(d[i][j]+";");
			}
			out.println("");
		}
		
    	String toWrite = buffer.toString();

    	
    	try {
    		FileWriter fw = new FileWriter( f );
    		fw.write( toWrite );
    		fw.close();
    	} catch ( IOException e ) {
      		System.out.println( "Konnte Datei nicht erstellen" );
    	}
	  
	}
	public static void writeTable(float[] d, File f) {
		StringWriter buffer = new StringWriter();
		PrintWriter out = new PrintWriter( buffer );
		for(int j = 0; j < d.length; j++) {	
			out.print(d[j]+";");
		}
		
		
    	String toWrite = buffer.toString();

    	
    	try {
    		FileWriter fw = new FileWriter( f );
    		fw.write( toWrite );
    		fw.close();
    	} catch ( IOException e ) {
      		System.out.println( "Konnte Datei nicht erstellen" );
    	}
	  
	}


}
