package ufz.aloe.urban.preprocessing;

import java.util.Vector;
import sa.data.*;
import sa.io.AsciiFileReader;
import sa.calc.*;
import sa.gui.*;
import ufz.aloe.urban.controler.UrbanModelControler;
import ufz.aloe.urban.model.*;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

public class Preprocessing {
	
	public static void main(String[] args) {
		if( args.length != 3 && args.length != 2 && args.length != 1) {
			System.err.println("usage: Preprocessing <inputfolder> <outputfolder> <id out>");
			System.err.println("or: Preprocessing -classpath");
			System.exit(0);
		}
		ProgressFrame frame=new ProgressFrame(null);
		AsciiFileReader asc;
		Raster boroughRaster;
		Raster structureType;
		Vector structureTypeData;
		Raster landParcelID;
		Vector householdTypeProportions;
		Vector cohorts;
		Vector borough;
		Vector buildings;
		Vector houses;
		String input="", output="";
		if(args.length==1 && !args[0].equals("-classpath")) {
			System.err.println("usage: Preprocessing <inputfolder> <outputfolder>");
			System.err.println("or: Preprocessing -classpath");
			System.exit(0);	
		}
		boolean idOut=true;
		if(args.length == 3) {
			if(args[0].endsWith("/")) input=args[0];
			else  input = args[0].concat("/");
			if(args[1].endsWith("/")) output=args[1];
			else  output=args[1].concat("/");
			if (args[2].toLowerCase().equals("false") 
				|| args[2].toLowerCase().equals("f")
				|| args[2].toLowerCase().equals("n")
				|| args[2].toLowerCase().equals("no") ) idOut=false;
		}
		asc = new AsciiFileReader(frame);
		boroughRaster = asc.readAsciiFile(input+"borough.asc", true);
		structureType = asc.readAsciiFile(input+"structureType.asc", true);
		buildings = UrbanModelControler.readParameterFiles(input+"buildingsPerParcel.csv");
		houses=UrbanModelControler.readParameterFiles(input+"houses.csv");
		structureTypeData = UrbanModelControler.readParameterFiles(input+"structureType.csv");
		borough = UrbanModelControler.readParameterFiles(input+"borough.csv");
		//landParcelID = asc.readAsciiFile(input+"landParcels.asc", true);
		householdTypeProportions = UrbanModelControler.readParameterFiles(input+"householdTypeProportions.csv");
		cohorts = UrbanModelControler.readParameterFiles(input+"cohorts.csv");
		float[] cohortProbability = City.getFloatArray(2,cohorts);
		float[] survival = City.getFloatArray(3,cohorts);
		float[] birthRate = City.getFloatArray(4,cohorts);
		float[][] householdProbability = City.getFloatMatrix(5,cohorts);
		float[] vacancy = City.getFloatArray(2,borough);
		float[][] cohortProportions = City.getFloatMatrix(3,borough);
		//float[][] build = City.getFloatMatrix(1,buildings);
		float[][] build = City.getFloatMatrix(1,houses);
		int[][] build_out = new int[build.length+2][build[0].length];
		
		System.out.print("deriving id ......");
		DeriveID did = new DeriveID();
		Raster id = did.deriveID(structureType, boroughRaster);		
		int maxValueID = (int)id.maxValue();
		System.out.println("  done");
		
		
		
		
		float[] buildingsPerParcel=new float[maxValueID+1];
		System.out.println(buildingsPerParcel.length);
		
		int noParcel=0;
		
		for(int i=0; i < build[0].length; i++) {
			int x= (int) ((build[0][i]-id.xll())/id.cellsize());
			int y= (int) ( id.sizeY()- (build[1][i]-id.yll())/id.cellsize());
			build_out[0][i]=i;
			build_out[1][i]=(int)build[0][i];
			build_out[2][i]=(int)build[1][i];
			build_out[3][i]=(int)id.valueAt(x,y);
			if( id.valueAt(x,y)!=id.noData()) {
				buildingsPerParcel[ (int) id.valueAt(x,y) ]++;
			}
			else noParcel++;
		}
		
		System.out.println(noParcel+" buildings without parcel");
	
		StructureType[] structureTypeArray;
		String [] structureTypes; //the use types that are available 
		int[] flatsPerBuilding; //the number of flats that a building has is a vector of structure type
		float[] numOfBuildings;
		float[] meanRadius;
		structureTypes = City.getStringArray(0,structureTypeData);
		flatsPerBuilding = City.getIntArray(2,structureTypeData);
		numOfBuildings = City.getFloatArray(3,structureTypeData);
		meanRadius = City.getFloatArray(6,structureTypeData);
		float[][] householdTypeProportionsMatrix =  City.getFloatMatrix(1, householdTypeProportions);
	//	System.out.println(householdTypeProportionsMatrix.length+"   "+householdTypeProportionsMatrix[0].length);
		if (structureTypes.length == flatsPerBuilding.length){
			structureTypeArray = new StructureType[structureTypes.length];
			for (int i=0; i<structureTypes.length; i++){
				StructureType b = new StructureType(structureTypes[i], flatsPerBuilding [i], i+1, meanRadius[i]);
				structureTypeArray[i] = b;
			}
		}
		else 
			throw new UnequalInputArrayLengthException("Problems with input arrays for use types");
	
	
		
		
		Vector parcels = new Vector();
		int maxID=0;
		
		
		System.out.print("calculating parcel distances....   ");
		PatchMetric metrics = new PatchMetric(id,false);
		//metrics.calcNearestPatchDistances();
		metrics.calcAveragePatchDistances();
		Raster landParcelDistances = metrics.patchDist;
		
		System.out.println("done");
		
		if(idOut) {
			System.out.print("writing dist raster ......");
			asc.writeAsciiFile(landParcelDistances, output+"landparcelDist2.asc");
			System.out.println("  done");
			System.out.print("writing id raster ......");
			asc.writeAsciiFile(id, output+"landparcelID.asc");
			System.out.println("  done");
		}
		
		System.out.print("calculating parcel areas ......");
		metrics.calcPatchAreas();
		System.out.println("  done");
		System.out.print("calculating parcel properties ......");
		float[] area = new float[maxValueID+1];	
		int[] initStructure = new int[maxValueID+1];
		int[] numBuildings = new int[maxValueID+1];
		int[] boroughArray = new int[maxValueID+1];
		for(int i=0; i < id.sizeY(); i++) {
			for(int j=0; j < id.sizeX(); j++) {
				//if(id.valueAt(i,j) > maxID) {
					maxID=(int) id.valueAt(i,j);
					if(maxID!=id.noData()){
						initStructure[maxID]=(int) structureType.valueAt(i,j);
						area[maxID] = (float) metrics.patchArea[maxID];
						numBuildings[maxID] = (int) buildingsPerParcel[maxID];
						boroughArray[maxID] = (int) boroughRaster.valueAt(i,j);
					}
					
			}
		}
		System.out.println("  done");
		
		
		System.out.print("writing parcel file ......");
		try {
			PrintWriter out;
			out = new PrintWriter(new BufferedWriter( 
					new FileWriter ("landparcel.csv")));
			out.print("LandParcelID;");
			for (int i=1; i<= maxValueID; i++){	
				if(i==maxValueID) out.print(i);
				else out.print(i+";");	
			}
			out.println("");
			out.print("Area;");
			for (int i=1; i< area.length; i++){	
				if(i==area.length-1) out.print(area[i]);
				else out.print(area[i]+";");	
			}
			out.println("");
			out.print("numberOfBuildings;");
			for (int i=1; i< numBuildings.length; i++){	
				if(i==numBuildings.length-1) out.print(numBuildings[i]);
				else out.print(numBuildings[i]+";");	
			}
			out.println("");
			out.print("initialStructureType;");
			for (int i=1; i< initStructure.length; i++){	
				if(i==initStructure.length-1) out.print(initStructure[i]);
				else out.print(initStructure[i]+";");	
			}
			out.println("");
			out.print("borough;");
			for (int i=1; i< boroughArray.length; i++){	
				if(i==boroughArray.length-1) out.print(boroughArray[i]);
				else out.print(boroughArray[i]+";");	
			}
			/*out.println("");
			for(int ht = 0; ht < householdTypeProportionsMatrix[0].length; ht++) {
				out.print("householdType"+(ht+1)+";");
				for (int i=1; i< initStructure.length; i++){	
					if(i==initStructure.length-1) out.print(householdTypeProportionsMatrix[initStructure[i]-1][ht]);
					else out.print(householdTypeProportionsMatrix[initStructure[i]-1][ht]+";");	
				}
				out.println("");
			}*/
		
			out.close();
		
		} catch (FileNotFoundException fnf) {
			System.err.println("File landparcel.csv not found! Please check if you forgot to " +
					"close this file. "+fnf);
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		System.out.print("writing transposed parcel file ......");
		try {
			PrintWriter out;
			out = new PrintWriter(new BufferedWriter( 
					new FileWriter ("landparcel_transp.csv")));
			out.println("LandParcelID;Area;numberOfBuildings;initialStructureType;borough;");
			for (int i=1; i<= maxValueID; i++){	
				out.println(i+";"+area[i]+";"+numBuildings[i]+";"+initStructure[i]+";"+boroughArray[i]);	
			}
		
			out.close();
		
		} catch (FileNotFoundException fnf) {
			System.err.println("File landparcel.csv not found! Please check if you forgot to " +
					"close this file. "+fnf);
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.out.println("  done");
		
		System.out.print("writing building file ......");
		try {
			PrintWriter out;
			out = new PrintWriter(new BufferedWriter( 
					new FileWriter ("houses_out.csv")));
			out.println("ID;X;Y;parcelID");
			for (int i=0; i< build_out[0].length; i++){	
				out.println(build_out[0][i]+";"+build_out[1][i]+";"+build_out[2][i]+";"+build_out[3][i]);	
			}
		
			out.close();
		
		} catch (FileNotFoundException fnf) {
			System.err.println("File landparcel.csv not found! Please check if you forgot to " +
					"close this file. "+fnf);
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.out.println("  done");
		
		
		System.exit(0);
	}	
}
